DECLARE SUB DrawLevelSpriteMap ()
DECLARE SUB DisplayTileInfo ()

DEFINT A-Z
'$INCLUDE: 'XMASCOM.BI'

DECLARE SUB RemoveSpriteAtCursor ()
DECLARE SUB EraseSprite (x%, y%)
DECLARE SUB DrawCursor (cursorColor%)
DECLARE SUB DrawPreview ()
DECLARE SUB DrawBackground (drawGrid%, redrawSprites%)
DECLARE SUB DrawSprite (xOffset%, yOffset%, spriteId%)
DECLARE SUB SelectDrawSprite ()
DECLARE SUB PlaceSpriteAtCursor ()
DECLARE SUB MoveCursor (xDelta%, yDelta%)
DECLARE SUB DrawBgGrid ()
DECLARE FUNCTION GetCursorModeStr$ ()
DECLARE SUB DisplayHelp ()
DECLARE SUB SaveLevel ()
DECLARE SUB LoadLevel ()
DECLARE SUB EditorExitScreen ()
DECLARE SUB ClearKbBuffer ()
DECLARE FUNCTION CheckFileExists% (filename$)
DECLARE SUB LoadSpriteIdFile ()
DECLARE SUB LoadSpriteIntoMap (spriteId%, filename$, SpriteMap%())


TYPE CursorType
    x AS INTEGER
    y AS INTEGER
    spriteId AS INTEGER
    mode AS INTEGER
END TYPE

TYPE TileInfo
    typeId AS INTEGER
    value AS INTEGER
    attributes AS STRING * 4
END TYPE

'Modes for cursor.
CONST LEVELMODE = 1
CONST ENEMYMODE = 2
CONST ITEMMODE = 3
CONST MARKERMODE = 4  'for start/end locations
CONST PREVIEWMODE = 5



DIM SHARED SpriteMap(MAXSPRITES, MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER
DIM SHARED SpriteIdFileNameMap(MAXSPRITES) AS STRING * 12
DIM SHARED LevelSpriteMap(MAXHORIZONTALSPRITES, MAXVERTICALSPRITES) AS INTEGER
DIM SHARED LevelTileInfoMap(MAXHORIZONTALSPRITES, MAXVERTICALSPRITES) AS TileInfo

DIM SHARED Cursor AS CursorType

Cursor.x = 1
Cursor.y = 1
Cursor.spriteId = 1
Cursor.mode = LEVELMODE


CALL LoadSpriteIdFile


SCREEN 13
CLS

CALL DrawBackground(TRUE, FALSE)
CALL DrawCursor(14)

quit% = FALSE

WHILE quit% = FALSE

    start! = TIMER

    in% = INP(96)
    LOCATE 3, 32
    PRINT "KEY:"; in%


    cursorXDelta% = 0
    cursorYDelta% = 0

    SELECT CASE in%

        CASE TABKEY
            Cursor.mode = Cursor.mode + 1
            IF Cursor.mode > MARKERMODE THEN Cursor.mode = 1
            CALL DrawBackground(FALSE, FALSE)


        CASE F1KEY
            CALL DisplayHelp

        CASE F2KEY
            CALL SaveLevel

        CASE F3KEY
            CALL LoadLevel
       
        CASE F5KEY
            CALL DrawPreview

        CASE DELKEY
            CALL RemoveSpriteAtCursor

        CASE UPARROW
            cursorYDelta% = -1
       
        CASE DOWNARROW
            cursorYDelta% = 1

        CASE LEFTARROW
            cursorXDelta% = -1

        CASE RIGHTARROW
            cursorXDelta% = 1

        CASE SPACEKEY
            CALL PlaceSpriteAtCursor

        CASE SKEY
            CALL SelectDrawSprite

        CASE QKEY
            quit% = TRUE

    END SELECT

    CALL ClearKbBuffer

    IF cursorYDelta% <> 0 OR cursorXDelta% <> 0 THEN
        CALL MoveCursor(cursorXDelta%, cursorYDelta%)
        CALL DisplayTileInfo
    END IF


    'fps control
    WHILE TIMER - start! <= .07: WEND

WEND

CALL EditorExitScreen


END

SUB DisplayHelp
END SUB

SUB DisplayTileInfo
   
    levelX% = Cursor.x / MAXSPRITEWIDTH + 1
    levelY% = Cursor.y / MAXSPRITEHEIGHT + 1
    COLOR 15
    LOCATE 11, 32
    PRINT "TILE INFO"
    LOCATE 12, 32
    PRINT "ID:"; LevelTileInfoMap(levelX%, levelY%).typeId
    LOCATE 13, 32
    PRINT SPACE$(8)
    LOCATE 13, 32
    PRINT "V:"; LevelTileInfoMap(levelX%, levelY%).value
    LOCATE 14, 32
    PRINT SPACE$(8)
    LOCATE 14, 32
    PRINT "A: "; LevelTileInfoMap(levelX%, levelY%).attributes

END SUB

SUB DrawBackground (drawGrid%, redrawSprites%)

    IF redrawSprites% THEN
        CALL DrawLevelSpriteMap
    END IF
  
    IF drawGrid% THEN
        CALL DrawBgGrid
    END IF


    COLOR 15
    LOCATE 22
    PRINT "Mode: "; GetCursorModeStr$

    LOCATE 5, 32
    PRINT "SPRITE"
    LOCATE 9, 32
    PRINT "("; Cursor.spriteId; ")"
    CALL DrawSprite(265, 42, Cursor.spriteId)

END SUB

SUB DrawBgGrid

    FOR yOffset% = 1 TO MAXLEVELHEIGHT STEP MAXSPRITEHEIGHT
        FOR xOffset% = 1 TO MAXLEVELWIDTH STEP MAXSPRITEWIDTH
   
            LINE (xOffset%, yOffset%)-(xOffset% + MAXSPRITEWIDTH, yOffset% + MAXSPRITEHEIGHT), 8, B

        NEXT xOffset%
    NEXT yOffset%



END SUB

SUB DrawCursor (cursorColor%)

    LINE (Cursor.x, Cursor.y)-(Cursor.x + MAXSPRITEWIDTH, Cursor.y + MAXSPRITEHEIGHT), cursorColor%, B

END SUB

SUB DrawLevelSpriteMap

    FOR mapY% = 1 TO UBOUND(LevelSpriteMap, 2)
        FOR mapX% = 1 TO UBOUND(LevelSpriteMap, 1)

            'need the upper left most corner + 1
            spriteDrawX% = mapX% * MAXSPRITEWIDTH - MAXSPRITEWIDTH
            spriteDrawY% = mapY% * MAXSPRITEHEIGHT - MAXSPRITEHEIGHT

            CALL DrawSprite(spriteDrawX%, spriteDrawY%, LevelSpriteMap(mapX%, mapY%))

        NEXT mapX%
    NEXT mapY%

END SUB

SUB DrawPreview

    CLS

    prevMode% = Cursor.mode
    Cursor.mode = PREVIEWMODE

    CALL DrawBackground(FALSE, TRUE)
    LOCATE 23
    COLOR 7
    PRINT "Press any key to return to editor"

    CALL ClearKbBuffer

    DO: LOOP WHILE INKEY$ = ""

    LOCATE 23
    PRINT SPACE$(33)

    Cursor.mode = prevMode%
   
    CALL DrawBackground(TRUE, FALSE)
   
END SUB

SUB DrawSprite (xOffset%, yOffset%, spriteId%)

    FOR y% = 1 TO MAXSPRITEHEIGHT
        FOR x% = 1 TO MAXSPRITEWIDTH

            'skips 'PSET'ing on blank sprite data to save draw time.

            IF SpriteMap(spriteId%, x%, y%) > 0 THEN
                PSET (x% + xOffset%, y% + yOffset%), SpriteMap(spriteId%, x%, y%)
            END IF

        NEXT x%
    NEXT y%

END SUB

SUB EraseSprite (x%, y%)

    'fill grid block with black then draw grid box

    LINE (x%, y%)-(x% + MAXSPRITEWIDTH, y% + MAXSPRITEHEIGHT), 0, BF
    LINE (x%, y%)-(x% + MAXSPRITEWIDTH, y% + MAXSPRITEHEIGHT), 8, B

END SUB

FUNCTION GetCursorModeStr$

    modeStr$ = "UNKNOWN"
   
    SELECT CASE Cursor.mode

        CASE LEVELMODE
            modeStr$ = "LEVEL  "

        CASE ENEMYMODE
            modeStr$ = "ENEMY  "

        CASE ITEMMODE
            modeStr$ = "ITEM   "

        CASE MARKERMODE
            modeStr$ = "MARK   "

        CASE PREVIEWMODE
            modeStr$ = "PREVIEW"

    END SELECT


    GetCursorModeStr$ = modeStr$

END FUNCTION

SUB LoadLevel

    LINE (20, 35)-(250, 50), 2, BF
    LINE (20, 50)-(250, 75), 7, BF
    COLOR 15
    LOCATE 6, 4
    PRINT "LOAD LEVEL"
    LOCATE 8, 4
    INPUT "FILENAME: ", filename$

    IF LTRIM$(RTRIM$(filename$)) = "" THEN
        CLS
        CALL DrawBackground(TRUE, TRUE)
        CALL DrawCursor(14)

        EXIT SUB
    END IF

    filename$ = UCASE$(LTRIM$(RTRIM$(filename$)))
    finalFileName$ = LEFT$(filename$, 8)

    dotIdx% = INSTR(filename$, ".")
   
    IF dotIdx% = FALSE THEN
         finalFileName$ = finalFileName$ + ".LVL"
    ELSE
        extension$ = RIGHT$(filename$, LEN(filename$) - dotIdx%)
        finalFileName$ = finalFileName$ + "." + LEFT$(extension$, 3)
    END IF

    OPEN finalFileName$ FOR INPUT AS #1
        FOR y% = 1 TO UBOUND(LevelSpriteMap, 2)
            FOR x% = 1 TO UBOUND(LevelSpriteMap, 1)
                INPUT #1, LevelSpriteMap(x%, y%)
            NEXT x%
        NEXT y%
        FOR y% = 1 TO UBOUND(LevelTileInfoMap, 2)
            FOR x% = 1 TO UBOUND(LevelTileInfoMap, 1)
                INPUT #1, LevelTileInfoMap(x%, y%).typeId, LevelTileInfoMap(x%, y%).value, LevelTileInfoMap(x%, y%).attributes
            NEXT x%
        NEXT y%
    CLOSE #1
      
    CLS
    CALL DrawBackground(TRUE, TRUE)
    CALL DrawCursor(14)

    LOCATE 23, 1
    PRINT "Loaded level: "; finalFileName$


END SUB

SUB LoadSpriteIdFile

    fileExists% = CheckFileExists%(SPRITEIDFILE)

    IF fileExists% THEN

        OPEN SPRITEIDFILE FOR INPUT AS #1

            WHILE NOT EOF(1)
     
                INPUT #1, tempId%, tempFileName$
 
                IF tempId% < MAXSPRITES THEN
                    SpriteIdFileNameMap(tempId%) = tempFileName$

                    IF LTRIM$(RTRIM$(tempFileName$)) <> "" THEN
                        CALL LoadSpriteIntoMap(tempId%, tempFileName$, SpriteMap())
                    END IF

                END IF
            WEND
        CLOSE #1

        LOCATE 22, 2
        COLOR 15
        PRINT "Loaded sprite id file: "; SPRITEIDFILE

    ELSE
        LOCATE 22, 2
        COLOR 15
        PRINT SPRITEIDFILE; " Doesn't exist to load!"

    END IF


END SUB

SUB MoveCursor (xDelta%, yDelta%)

    IF xDelta% = 0 AND yDelta% = 0 THEN
        EXIT SUB
    END IF


    IF xDelta% > 0 THEN
        xDelta% = MAXSPRITEWIDTH
    ELSEIF xDelta% < 0 THEN
        xDelta% = -MAXSPRITEWIDTH
    END IF


    IF yDelta% > 0 THEN
        yDelta% = MAXSPRITEHEIGHT
    ELSEIF yDelta% < 0 THEN
        yDelta% = -MAXSPRITEHEIGHT
    END IF

    'draw cursor mask at old location.
    CALL DrawCursor(8)

    startX% = Cursor.x
    startY% = Cursor.y

    Cursor.x = Cursor.x + xDelta%
    Cursor.y = Cursor.y + yDelta%

    IF Cursor.x > MAXLEVELWIDTH OR Cursor.y > MAXLEVELHEIGHT OR Cursor.x < 1 OR Cursor.y < 1 THEN
        Cursor.x = startX%
        Cursor.y = startY%
    END IF

    CALL DrawCursor(14)

END SUB

SUB PlaceSpriteAtCursor

    IF Cursor.spriteId < 1 OR Cursor.spriteId > MAXSPRITES THEN
        EXIT SUB
    END IF

    CALL EraseSprite(Cursor.x, Cursor.y)
    CALL DrawSprite(Cursor.x, Cursor.y, Cursor.spriteId)

    levelX% = Cursor.x / MAXSPRITEWIDTH + 1
    levelY% = Cursor.y / MAXSPRITEHEIGHT + 1
    
    LevelSpriteMap(levelX%, levelY%) = Cursor.spriteId

    LevelTileInfoMap(levelX%, levelY%).typeId = Cursor.mode

    IF Cursor.mode <> LEVELMODE THEN

        CALL ClearKbBuffer

        LINE (20, 35)-(200, 50), 2, BF
        LINE (20, 50)-(200, 75), 7, BF
        COLOR 15
        LOCATE 6, 4
        PRINT "NEW SPRITE ATTRIBUTES"

        LOCATE 8, 4
        INPUT "Attribute: ", attrVal$
        LevelTileInfoMap(levelX%, levelY%).attributes = UCASE$(attrVal$)

        LOCATE 9, 4
        INPUT "Value: ", newVal$
        LevelTileInfoMap(levelX%, levelY%).value = VAL(newVal$)
      
        CLS
        CALL DrawBackground(TRUE, TRUE)
    ELSE
        LevelTileInfoMap(levelX%, levelY%).value = 0
        LevelTileInfoMap(levelX%, levelY%).attributes = BLOCKINGATTR
    END IF
     
    CALL DrawCursor(14)
    CALL DisplayTileInfo

    COLOR 7
    LOCATE 23
    PRINT "Placed: "; levelX%; ","; levelY%

END SUB

SUB RemoveSpriteAtCursor
   
    CALL EraseSprite(Cursor.x, Cursor.y)
   
    levelX% = Cursor.x / MAXSPRITEWIDTH + 1
    levelY% = Cursor.y / MAXSPRITEHEIGHT + 1
   
    LevelSpriteMap(levelX%, levelY%) = 0
    LevelTileInfoMap(levelX%, levelY%).typeId = TILETYPEUNSET
    LevelTileInfoMap(levelX%, levelY%).value = 0
    LevelTileInfoMap(levelX%, levelY%).attributes = ""

    CALL DrawCursor(14)
    CALL DisplayTileInfo

    COLOR 7
    LOCATE 23
    PRINT "Removed: "; levelX%; ","; levelY%



END SUB

SUB SaveLevel

    LINE (20, 35)-(250, 50), 2, BF
    LINE (20, 50)-(250, 75), 7, BF
    COLOR 15
    LOCATE 6, 4
    PRINT "SAVE LEVEL"
    LOCATE 8, 4
    INPUT "FILENAME: ", filename$

    IF LTRIM$(RTRIM$(filename$)) = "" THEN
        CLS
        CALL DrawBackground(TRUE, TRUE)
        CALL DrawCursor(14)

        EXIT SUB
    END IF

    filename$ = UCASE$(LTRIM$(RTRIM$(filename$)))
    finalFileName$ = LEFT$(filename$, 8)

    dotIdx% = INSTR(filename$, ".")
    
    IF dotIdx% = FALSE THEN
         finalFileName$ = finalFileName$ + ".LVL"
    ELSE
        extension$ = RIGHT$(filename$, LEN(filename$) - dotIdx%)
        finalFileName$ = finalFileName$ + "." + LEFT$(extension$, 3)
    END IF

    OPEN finalFileName$ FOR OUTPUT AS #1
        FOR y% = 1 TO UBOUND(LevelSpriteMap, 2)
            FOR x% = 1 TO UBOUND(LevelSpriteMap, 1)
                WRITE #1, LevelSpriteMap(x%, y%)
            NEXT x%
        NEXT y%
        FOR y% = 1 TO UBOUND(LevelTileInfoMap, 2)
            FOR x% = 1 TO UBOUND(LevelTileInfoMap, 1)

               
                'For player markers and enemy types , if value is not set,
                'use sprite id. Sprite id in value is used during
                'loading in the engine to set start of animation frames

                IF LevelTileInfoMap(x%, y%).typeId = TILETYPEMARKER AND LevelTileInfoMap(x%, y%).value = 0 AND LevelTileInfoMap(x%, y%).attributes = PLAYERATTR THEN
                     LevelTileInfoMap(x%, y%).value = LevelSpriteMap(x%, y%)
               
                ELSEIF LevelTileInfoMap(x%, y%).typeId = TILETYPEENEMY AND LevelTileInfoMap(x%, y%).value = 0 THEN
                     LevelTileInfoMap(x%, y%).value = LevelSpriteMap(x%, y%)
                END IF


                WRITE #1, LevelTileInfoMap(x%, y%).typeId, LevelTileInfoMap(x%, y%).value, LevelTileInfoMap(x%, y%).attributes
            NEXT x%
        NEXT y%
    CLOSE #1
       
    CLS
    CALL DrawBackground(TRUE, TRUE)
    CALL DrawCursor(14)

    LOCATE 23, 1
    PRINT "Saved level: "; finalFileName$


END SUB

SUB SelectDrawSprite

    CLS

    sid% = 1

    CALL DrawBgGrid

    FOR yOffset% = 1 TO MAXLEVELHEIGHT STEP MAXSPRITEHEIGHT
        FOR xOffset% = 1 TO MAXLEVELWIDTH STEP MAXSPRITEWIDTH
  
            IF sid% >= MAXSPRITES THEN EXIT FOR

            CALL DrawSprite(xOffset%, yOffset%, sid%)

            sid% = sid% + 1

        NEXT xOffset%
    NEXT yOffset%

    'save map cursor positions and reset for sprite selection.
    cursorPrevX% = Cursor.x
    cursorPrevY% = Cursor.y
    Cursor.x = 1
    Cursor.y = 1

    CALL DrawCursor(14)

    CALL ClearKbBuffer

    in% = 0
    quit% = FALSE

    WHILE quit% = FALSE

        start! = TIMER

        in% = INP(96)

        cursorXDelta% = 0
        cursorYDelta% = 0

        SELECT CASE in%

            CASE ESCKEY
                quit% = TRUE

            CASE ENTERKEY
                mapX% = Cursor.x / MAXSPRITEWIDTH + 1
                mapY% = Cursor.y / MAXSPRITEHEIGHT + 1

                selectedId% = MAXHORIZONTALSPRITES * (mapY% - 1) + mapX%

                IF selectedId% <= MAXSPRITES AND selectedId% > 0 THEN
                    Cursor.spriteId = selectedId%
                END IF

                quit% = TRUE

            CASE UPARROW
                cursorYDelta% = -1
      
            CASE DOWNARROW
                cursorYDelta% = 1

            CASE LEFTARROW
                cursorXDelta% = -1

            CASE RIGHTARROW
                cursorXDelta% = 1

        END SELECT

        CALL MoveCursor(cursorXDelta%, cursorYDelta%)
        CALL ClearKbBuffer
         
        'fps control
        WHILE TIMER - start! <= .07: WEND


    WEND
   


    'set back prev values before returning.
    Cursor.x = cursorPrevX%
    Cursor.y = cursorPrevY%

    CLS
    CALL DrawBackground(TRUE, TRUE)
    CALL DrawCursor(14)

END SUB

