DEFINT A-Z
'$INCLUDE: 'XMASCOM.BI'

DECLARE FUNCTION CheckFileExists% (fileName$)
DECLARE SUB LoadSpriteIntoMap (spriteId%, fileName$, SpriteMap%())
DECLARE SUB EditorExitScreen ()
DECLARE SUB ClearKbBuffer ()
DECLARE SUB DisplayHelp ()
DECLARE SUB LoadSpriteIdMap ()
DECLARE SUB DrawSprite (xOffset%, yOffset%, spriteId%)
DECLARE SUB SetFileNameForIdx ()
DECLARE SUB PrintSpriteIdMap (startIdx%)
DECLARE SUB SaveSpriteIdMap ()
DECLARE SUB DrawBackground ()



DIM SHARED SpriteMap(MAXSPRITES, MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER
DIM SHARED SpriteIdFileNameMap(MAXSPRITES) AS STRING * 12

CONST PAGESIZE = 6


SCREEN 13


CLS

CALL DrawBackground
CALL LoadSpriteIdMap


quit% = FALSE
currentPage% = 0
startIdx% = 1


WHILE quit% = FALSE

    start! = TIMER

    in% = INP(96)
    LOCATE 3, 35
    PRINT in%

    SELECT CASE in%

        CASE F1KEY
            CALL DisplayHelp

        CASE F2KEY
            CALL SaveSpriteIdMap

        CASE F3KEY
            CALL LoadSpriteIdMap

        CASE PGDOWN
            currentPage% = currentPage% + 1
            IF (currentPage% * PAGESIZE) > MAXSPRITES THEN
                currentPage% = currentPage% - 1
            END IF
            startIdx% = (currentPage% * PAGESIZE) - 1


        CASE PGUP
            currentPage% = currentPage% - 1
            IF currentPage% < 0 THEN currentPage% = 0
            IF currentPage% > 1 THEN
                startIdx% = (currentPage% * PAGESIZE) - 1
            ELSE
                startIdx% = 1
            END IF

        CASE UPARROW
            startIdx% = startIdx% - 1
            IF startIdx% < 1 THEN startIdx% = 1
            IF startIdx% MOD PAGESIZE = 0 THEN
                currentPage% = currentPage% - 1
                IF currentPage% < 0 THEN currentPage% = 0
            END IF


        CASE DOWNARROW
            startIdx% = startIdx% + 1
            IF startIdx% >= MAXSPRITES - PAGESIZE THEN
                startIdx% = MAXSPRITES - PAGESIZE + 1
            END IF
            IF startIdx% MOD PAGESIZE = 0 THEN
                currentPage% = currentPage% + 1
            END IF


        CASE IKEY
            CALL SetFileNameForIdx

        CASE QKEY
            quit% = TRUE



    END SELECT

    CALL ClearKbBuffer


    CALL PrintSpriteIdMap(startIdx%)
    

    'fps control
    WHILE TIMER - start! <= .05: WEND


WEND

CALL EditorExitScreen

END

SUB DisplayHelp

    CALL DrawBackground
    CALL ClearKbBuffer

    COLOR 7
    LOCATE 5, 10
    PRINT "KEYBOARD COMMANDS"
    LOCATE 6, 9
    PRINT ""
    PRINT "   F1 - Show this help"
    PRINT "   F2 - Save current file mappings"
    PRINT "   F3 - Load existing file mappings"
    PRINT "    I - Set file name for ID"
    PRINT "    Q - Quit program"
    PRINT "   UP - Scroll up "
    PRINT " DOWN - Scroll down"
    PRINT "PG UP - Scroll one page up"
    PRINT "PG DN - Scroll one page down"
    PRINT
    PRINT "Press any key to return..."


    DO: LOOP WHILE INKEY$ = ""

    CALL DrawBackground


END SUB

DEFSNG A-Z
SUB DrawBackground
   
    CLS

    COLOR 2
    PRINT ""
    PRINT ""
    
    COLOR 15
    LOCATE 1, 8
    PRINT "CHRISTMAS 2021 QB CHALLENGE"
    LOCATE 2, 12
    PRINT "SPRITE ID EDITOR"



END SUB

DEFINT A-Z
SUB DrawSprite (xOffset%, yOffset%, spriteId%)

  
    LINE (xOffset%, yOffset%)-(xOffset% + MAXSPRITEWIDTH + 1, yOffset% + MAXSPRITEHEIGHT + 1), 15, B


    FOR y% = 1 TO MAXSPRITEHEIGHT
        FOR x% = 1 TO MAXSPRITEWIDTH
            PSET (x% + xOffset%, y% + yOffset%), SpriteMap(spriteId%, x%, y%)
        NEXT x%
    NEXT y%


END SUB

SUB LoadSpriteIdMap

    fileExists% = CheckFileExists%(SPRITEIDFILE)

    IF fileExists% THEN

        OPEN SPRITEIDFILE FOR INPUT AS #1

            WHILE NOT EOF(1)
         
                INPUT #1, tempId%, tempFileName$
     
                IF tempId% < MAXSPRITES THEN
                    SpriteIdFileNameMap(tempId%) = tempFileName$

                    IF LTRIM$(RTRIM$(tempFileName$)) <> "" THEN
                        CALL LoadSpriteIntoMap(tempId%, tempFileName$, SpriteMap())
                    END IF

                END IF
            WEND
        CLOSE #1

        LOCATE 22, 2
        COLOR 15
        PRINT "Loaded sprite id file: "; SPRITEIDFILE

    ELSE
        LOCATE 22, 2
        COLOR 15
        PRINT SPRITEIDFILE; " Doesn't exist to load!"

    END IF



END SUB

SUB PrintSpriteIdMap (startIdx%)


    COLOR 7

    curIdx% = 1
    IF startIdx% > 0 THEN
        'curIdx% = (pageNum% * PAGESIZE) - 1
        curIdx% = startIdx%
    END IF

    LOCATE 4, 30
    PRINT "idx:"; curIdx%

    FOR y% = 4 TO 21 STEP 3

        LOCATE y%, 5
        IF curIdx% <= MAXSPRITES THEN
            PRINT curIdx%; " - "; SpriteIdFileNameMap(curIdx%)

            'nice little magic numbers here... :)
            CALL DrawSprite(10, (y% * 8) - 15, curIdx%)

        ELSE
            LOCATE y%, 1
            PRINT SPACE$(30)
        END IF

        curIdx% = curIdx% + 1

   NEXT y%


END SUB

SUB SaveSpriteIdMap

    OPEN SPRITEIDFILE FOR OUTPUT AS #1

        FOR i% = 1 TO UBOUND(SpriteIdFileNameMap)

            WRITE #1, i%, SpriteIdFileNameMap(i%)
     
        NEXT i%
    CLOSE #1

    LOCATE 22, 2
    COLOR 15
    PRINT "Saved sprite id file: "; SPRITEIDFILE; "  "

END SUB

SUB SetFileNameForIdx

    CALL ClearKbBuffer

    LOCATE 10, 2
    COLOR 2
    PRINT ""

    LOCATE 10, 7
    COLOR 15
    PRINT "SET FILE NAME FOR ID"

    COLOR 7
    LOCATE 11, 2
    PRINT ""
    LOCATE 12, 2
    PRINT ""
    LOCATE 13, 2
    PRINT ""
    LOCATE 14, 2
    PRINT ""
    LOCATE 15, 2
    PRINT ""



    LOCATE 12, 3
    COLOR 15
    INPUT "Sprite ID: ", spriteId%

    IF spriteId% < 1 OR spriteId% > MAXSPRITES THEN
        CLS
        CALL DrawBackground
        EXIT SUB
    END IF


    LOCATE 13, 3
    INPUT "File name: ", spriteFileName$

    IF LTRIM$(RTRIM$(spriteFileName$)) = "" THEN
        SpriteIdFileNameMap(spriteId%) = ""
        CALL LoadSpriteIntoMap(spriteId%, spriteFileName$, SpriteMap())
        CLS
        CALL DrawBackground
        EXIT SUB
    END IF

    IF INSTR(spriteFileName$, ".") = 0 THEN
        fileName$ = MID$(spriteFileName$, 1, 8)
        spriteFileName$ = fileName$ + ".SDX"
    END IF


    IF CheckFileExists%(spriteFileName$) = FALSE THEN
        LOCATE 14, 3
        COLOR 4
        PRINT "ERROR: FILE NOT FOUND"
        DO: LOOP WHILE INKEY$ = ""

        CLS
        CALL DrawBackground
        EXIT SUB
    END IF

   
    SpriteIdFileNameMap(spriteId%) = spriteFileName$


    CALL LoadSpriteIntoMap(spriteId%, spriteFileName$, SpriteMap())

    CLS
    CALL DrawBackground


    LOCATE 23, 2
    'COLOR 15, 2
    PRINT "Updated: "; spriteFileName$
END SUB

