DECLARE SUB CheckEnemyPlayerCollisions ()
DECLARE FUNCTION IsBlockingMarkerTile% (levelX%, levelY%)
DECLARE FUNCTION MainMenu% (prevMenuChoice%)
DECLARE SUB DrawSpriteMask (xOffset%, yOffset%)
DECLARE SUB ToggleSound ()
DECLARE SUB DrawLevelSpriteMap (drawLevelOnly%, drawPlayerStart%)
DECLARE SUB SetKeyState ()
DECLARE SUB HandleHighScore ()
DECLARE SUB HighScores ()
DECLARE SUB HandleLevelComplete ()
DECLARE SUB GenPresentColors ()
DECLARE SUB EndOfDemo ()
DECLARE SUB ExitScreen ()
DECLARE SUB PlaySfx (sfxId%)
DECLARE SUB Instructions ()
DECLARE SUB EditorExitScreen ()
DECLARE SUB RunLevel ()
DECLARE SUB Init (LevelFileName$)
DECLARE SUB DrawStatusInfo (redrawSpiritBar%, redrawLives%, redrawDeliverCount%, redrawPresent%, redrawItem%, redrawMode%)

' TODO : there's a lot of copy/pasted code from LVL_EDIT instead
'        of being used in a common module. This should be moved
'        if possible... well there was.. a lot is modified now. :)

DEFINT A-Z
'$INCLUDE: 'XMASCOM.BI'

DECLARE SUB CheckPickUpItem (levelX%, levelY%)
DECLARE SUB DrawFgTiles ()
DECLARE SUB HandleEnemyAttack (enemyIdx%)
DECLARE FUNCTION CheckEnemyCollisions% (enemyId%, deltaX%)
DECLARE SUB DrawEnemy (enemyIdx%)
DECLARE SUB MoveEnemies ()
DECLARE SUB CheckPickUpPresent (levelX%, topLevelY%)
DECLARE SUB CheckDropOffPresent (levelX%, topLevelY%)
DECLARE SUB DrawBackground ()
DECLARE SUB HandleDeath ()
DECLARE SUB HandleLvlTimer ()
DECLARE SUB ChangePlayerHealth (healthDelta%)
DECLARE SUB DecreasePlayerHealth (decreaseAmount%)
DECLARE SUB DrawPlayer ()
DECLARE SUB DrawDebugGrid ()
DECLARE FUNCTION IsBlockingSprite% (x%, y%)
DECLARE FUNCTION IsOnGround% (x%, y%)
DECLARE SUB MovePlayer (deltaX%, deltaY%)
DECLARE SUB SetSpriteForPut (SpriteId%, destSprite%())
DECLARE SUB DrawSprite (xOffset%, yOffset%, SpriteId%)
DECLARE SUB LoadLevel (LevelFileName$)
DECLARE SUB LoadSpriteIdFile ()
DECLARE SUB LoadSpriteIntoMap (SpriteId%, fileName$, SpriteMap%())
DECLARE FUNCTION CheckFileExists% (fileName$)
DECLARE SUB ClearKbBuffer ()

CONST VERSION = "0.28-DEMO"

CONST DEFAULTPLAYERSTEP = 2
CONST MAXJUMPHEIGHT = MAXSPRITEHEIGHT * 2.5

CONST OUTOFBOUNDSY = 144
CONST OUTOFBOUNDSX = 224

CONST FORWARD = 0
CONST BACKWARD = 1
CONST STARTLIVES = 5
CONST MAXHEALTH = 100
CONST HEALTHDELIVERYBONUS = 15
CONST HEALTHPICKUPBONUS = 8

CONST MAXENEMIES = 15
CONST DEFAULTENEMYSLEEPDURATION = 50
CONST DEFAULTENEMYSTEP = 1
CONST ENEMYMOVEFRAME = 1


CONST MAXITEMS = 15


CONST MENUSELECTSFX = 0
CONST JUMPSFX = 1
CONST PICKUPITEMSFX = 2
CONST BLENDSFX = 3
CONST RUNSFX = 4
CONST WALKSFX = 5
CONST UNBLENDSFX = 6
CONST PICKUPPRESENTSFX = 7
CONST DROPOFFPRESENTSFX = 8
CONST PLAYERDIEDSFX = 9
CONST PLAYERHURTSFX = 10
CONST LEVELCOMPLETESFX = 11
CONST GAMEOVERSFX = 12


CONST DELIVEREDPRESENTSCORE = 250
CONST TIMESCORE = .5
CONST LIVESSCORE = 100


CONST HIGHSCOREFILE = "HIGHSCOR.HDX"
CONST MAXHIGHSCORES = 10

CONST STATEIDLEFTARROW = 0
CONST STATEIDRIGHTARROW = 1
CONST STATEIDSPACEKEY = 2
CONST STATEIDRKEY = 3
CONST STATEIDQKEY = 4
CONST STATEIDBKEY = 5
CONST STATEIDZKEY = 6
CONST STATEIDSKEY = 7


RANDOMIZE TIMER

TYPE SpriteFrameInfo
    FirstSpriteId AS INTEGER
    BlendSpriteId AS INTEGER
    Direction AS INTEGER
    forwardFrameNum AS INTEGER
    backwardFrameNum AS INTEGER
END TYPE


TYPE TileInfo
    TypeId AS INTEGER
    value AS INTEGER
    attributes AS STRING * 4
END TYPE


TYPE LevelInfoType
    LevelNum AS INTEGER
    LevelFileName AS STRING * 10
    PlayerStartX AS INTEGER
    PlayerStartY AS INTEGER
    StartTimerVal AS INTEGER
    CurTimerVal AS INTEGER
    NumEnemies AS INTEGER
    NumItems AS INTEGER
    NumPresents AS INTEGER
    NumPresentsDelivered AS INTEGER
    CurPresentFgColor AS INTEGER
    CurPresentBgColor AS INTEGER
    LevelComplete AS INTEGER
END TYPE

TYPE EnemyType
    x AS INTEGER
    y AS INTEGER
    MaxSleepTicks AS INTEGER     'max sleep ticks allowed
    CurMaxSleepTicks AS INTEGER  'current max of 'this' sleep
    CurSleepTicks AS INTEGER     'current ticks of 'this' sleep
    isActive AS INTEGER
    IsRemoved AS INTEGER
    startX AS INTEGER
    startY AS INTEGER
    CurEnemyStep AS INTEGER
    SpriteFrame AS SpriteFrameInfo
END TYPE


TYPE PlayerType
    x AS INTEGER
    y AS INTEGER
    levelX AS INTEGER
    levelY AS INTEGER
    health AS INTEGER
    Lives AS INTEGER
    StepSize AS INTEGER
    IsGameOver AS INTEGER
    HasPresent AS INTEGER
    hasItem AS INTEGER
    itemId AS INTEGER
    IsJumping AS INTEGER
    maxJumpY AS INTEGER
    IsOnGround AS INTEGER
    IsBlending AS INTEGER
    IsRunning AS INTEGER
    Score AS LONG
    SpriteFrame AS SpriteFrameInfo
END TYPE


TYPE ItemType
    levelX AS INTEGER
    levelY AS INTEGER
    x AS INTEGER
    y AS INTEGER
    value AS INTEGER
    IsPickedUp AS INTEGER
    SpriteId AS INTEGER
    attribute AS STRING * 4
END TYPE


TYPE HighScoreData
    DateStr AS STRING * 10
    PlayerName AS STRING * 16
    Score AS LONG
END TYPE


DIM SHARED SpriteMap(MAXSPRITES, MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER
DIM SHARED SpriteIdFileNameMap(MAXSPRITES) AS STRING * 12
DIM SHARED LevelSpriteMap(MAXHORIZONTALSPRITES, MAXVERTICALSPRITES) AS INTEGER
DIM SHARED LevelTileInfoMap(MAXHORIZONTALSPRITES, MAXVERTICALSPRITES) AS TileInfo

DIM SHARED MaskSprite(MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER

DIM SHARED PlayerSpriteF0(MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER
DIM SHARED PlayerSpriteF1(MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER
DIM SHARED PlayerSpriteB0(MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER
DIM SHARED PlayerSpriteB1(MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER
DIM SHARED PlayerSpriteBlend(MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER


'Using enemy start sprite + offset and DrawSprite instead of PUT
'DIM SHARED EnemySpriteF0(MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER
'DIM SHARED EnemySpriteF1(MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER
'DIM SHARED EnemySpriteB0(MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER
'DIM SHARED EnemySpriteB1(MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER


DIM SHARED Player AS PlayerType
DIM SHARED Enemy(MAXENEMIES) AS EnemyType
DIM SHARED Item(MAXITEMS) AS ItemType
DIM SHARED LevelInfo AS LevelInfoType

DIM SHARED KeyState(7) AS INTEGER
DIM SHARED LastKeyStateId AS INTEGER

DIM SHARED inPrintY AS INTEGER
inPrintY = 19

DIM SHARED SoundOn AS INTEGER

CLS
SCREEN 13

CALL LoadSpriteIdFile 'different sprite maps aren't used. this only needs
                      'to be called once.

SoundOn = TRUE
quit% = FALSE
menuChoice% = 0



WHILE quit% = FALSE
    menuChoice% = MainMenu%(menuChoice%)
   
    SELECT CASE menuChoice%

        CASE 0
            CALL PlaySfx(MENUSELECTSFX)

            Player.IsGameOver = FALSE
            Player.Lives = STARTLIVES
            Player.Score = 0

            LevelInfo.LevelNum = 0

            WHILE Player.IsGameOver = FALSE AND LevelInfo.LevelNum <= 2 ' for demo
                                     
                LevelInfo.LevelFileName = "LEVEL" + LTRIM$(RTRIM$(STR$(LevelInfo.LevelNum))) + ".LVL"
                'LevelInfo.LevelFileName = "TEST" + LTRIM$(RTRIM$(STR$(LevelInfo.LevelNum))) + ".LVL"
                

                CALL Init(LevelInfo.LevelFileName)
                CALL RunLevel
            WEND

            IF LevelInfo.LevelNum > 2 THEN
                CALL EndOfDemo
            END IF

            CALL HandleHighScore



        CASE 1
            CALL PlaySfx(MENUSELECTSFX)
            CALL Instructions

        CASE 2
            CALL PlaySfx(MENUSELECTSFX)
            CALL HighScores

        CASE 3
            CALL ToggleSound
            CALL PlaySfx(MENUSELECTSFX)

        CASE 4
            CALL PlaySfx(MENUSELECTSFX)
            quit% = TRUE


    END SELECT

WEND

'CLEAR , , 2000
CLEAR
CALL ExitScreen


END

SUB ChangePlayerHealth (healthDelta%)


    Player.health = Player.health + healthDelta%

    IF Player.health > MAXHEALTH THEN
        Player.health = MAXHEALTH
    END IF

    CALL DrawStatusInfo(TRUE, FALSE, FALSE, FALSE, FALSE, FALSE)

   
    IF Player.health <= 0 THEN

        CALL HandleDeath
    END IF

   
END SUB

SUB CheckDropOffPresent (levelX%, topLevelY%)

    IF LevelTileInfoMap(levelX%, topLevelY%).attributes = TREEATTR AND LevelTileInfoMap(levelX%, topLevelY%).TypeId = TILETYPEMARKER THEN
          
     
        IF Player.HasPresent = TRUE THEN
            Player.HasPresent = FALSE

            LevelInfo.NumPresentsDelivered = LevelInfo.NumPresentsDelivered + 1

            CALL ChangePlayerHealth(HEALTHDELIVERYBONUS)

            CALL DrawStatusInfo(FALSE, FALSE, TRUE, TRUE, FALSE, FALSE)

            CALL PlaySfx(DROPOFFPRESENTSFX)

           
            ' Player resets at the drop off now if they die.
            LevelInfo.PlayerStartX = Player.x
            LevelInfo.PlayerStartY = Player.y
           
        END IF
    END IF


END SUB

FUNCTION CheckEnemyCollisions% (enemyIdx%, deltaX%)

    'Function returns TRUE on collision with blocking object.
    'Annoyingly enough, the action based on this collision is
    'handled here as well.
    'Thankfully, enemies currently only move on the X-axis so
    'it makes things a little simpler.


    CheckEnemyCollisions% = FALSE

    leftLevelX% = INT((Enemy(enemyIdx%).x) / MAXSPRITEWIDTH) + 1
    rightLevelX% = INT((Enemy(enemyIdx%).x + MAXSPRITEWIDTH - 1) / MAXSPRITEWIDTH) + 1
  
   
    topLevelY% = INT(Enemy(enemyIdx%).y / MAXSPRITEHEIGHT) + 1
    bottomLevelY% = INT((Enemy(enemyIdx%).y + MAXSPRITEHEIGHT) / MAXSPRITEHEIGHT) + 1


    ' Check if there's a ledge in front of the enemy's walk path
    IF bottomLevelY% <= UBOUND(LevelTileInfoMap, 2) AND bottomLevelY% > 0 THEN

        IF Enemy(enemyIdx%).SpriteFrame.Direction = BACKWARD AND LevelTileInfoMap(leftLevelX%, bottomLevelY%).TypeId <> TILETYPEBLOCKING THEN
            CheckEnemyCollisions% = TRUE
            EXIT FUNCTION
     
        ELSEIF Enemy(enemyIdx%).SpriteFrame.Direction = FORWARD AND LevelTileInfoMap(rightLevelX%, bottomLevelY%).TypeId <> TILETYPEBLOCKING THEN
            CheckEnemyCollisions% = TRUE
            EXIT FUNCTION
        
        END IF
    END IF



    ' Check right wall collisions and set sprite dir.
    IF deltaX% > 0 THEN

        Enemy(enemyIdx%).SpriteFrame.Direction = FORWARD

        checkLevelX% = INT((Enemy(enemyIdx%).x + MAXSPRITEWIDTH - 1 + deltaX%) / MAXSPRITEWIDTH) + 1

        'check level boundary.
        IF checkLevelX% < LBOUND(LevelTileInfoMap, 1) OR checkLevelX% > UBOUND(LevelTileInfoMap, 1) THEN
            CheckEnemyCollisions% = TRUE
            EXIT FUNCTION

        ELSE
            IF LevelTileInfoMap(checkLevelX%, topLevelY%).TypeId = TILETYPEBLOCKING THEN ' OR LevelTileInfoMap(checkLevelX%, bottomLevelY%).typeId = TILETYPEBLOCKING THEN
                CheckEnemyCollisions% = TRUE
                EXIT FUNCTION

            'check blocking marker sprites
            ELSEIF LevelTileInfoMap(checkLevelX%, topLevelY%).TypeId = TILETYPEMARKER THEN

                IF IsBlockingMarkerTile%(checkLevelX%, topLevelY%) THEN
                    CheckEnemyCollisions% = TRUE
                    EXIT FUNCTION
                END IF

            END IF
        END IF

         
    END IF


    ' Check left wall collisions and set sprite dir.
    IF deltaX% < 0 THEN

        Enemy(enemyIdx%).SpriteFrame.Direction = BACKWARD

        checkLevelX% = INT((Enemy(enemyIdx%).x + deltaX%) / MAXSPRITEWIDTH) + 1
    
        'check level boundary.
        IF checkLevelX% < LBOUND(LevelTileInfoMap, 1) OR checkLevelX% > UBOUND(LevelTileInfoMap, 1) THEN
            CheckEnemyCollisions% = TRUE
            EXIT FUNCTION
       
        ELSE
            IF LevelTileInfoMap(checkLevelX%, topLevelY%).TypeId = TILETYPEBLOCKING THEN ' OR LevelTileInfoMap(checkLevelX%, bottomLevelY%).typeId = TILETYPEBLOCKING THEN
                CheckEnemyCollisions% = TRUE
                EXIT FUNCTION

            'check blocking marker sprites
            ELSEIF LevelTileInfoMap(checkLevelX%, topLevelY%).TypeId = TILETYPEMARKER THEN

                IF IsBlockingMarkerTile%(checkLevelX%, topLevelY%) THEN
                    CheckEnemyCollisions% = TRUE
                    EXIT FUNCTION
                END IF

            END IF
        END IF

    END IF


    'Check collision with player
    'CheckEnemyCollisions% = CheckEnemyPlayerCollision%(enemyIdx%)
        
    

END FUNCTION

SUB CheckEnemyPlayerCollisions


    'Check enemy collisions with player and handles attack
   
       
    ' exit early if impossible to collide

    IF Player.IsBlending THEN EXIT SUB
   
    IF LevelInfo.NumEnemies <= 0 THEN EXIT SUB

  
    FOR i% = 0 TO LevelInfo.NumEnemies
        IF Enemy(i%).isActive AND Enemy(i%).IsRemoved = FALSE THEN

           
            IF Player.y + MAXSPRITEHEIGHT > Enemy(i%).y AND Player.y < Enemy(i%).y + MAXSPRITEHEIGHT THEN
      
                ' -4 on sprite width for collision detection grace.

                IF Player.x < Enemy(i%).x + (MAXSPRITEWIDTH - 4) AND Player.x + (MAXSPRITEWIDTH - 4) > Enemy(i%).x THEN
                    CALL HandleEnemyAttack(i%)
                END IF

              
            END IF
        END IF
    NEXT i%


END SUB

SUB CheckPickUpItem (levelX%, levelY%)

    IF LevelTileInfoMap(levelX%, levelY%).TypeId <> TILETYPEITEM OR LevelTileInfoMap(levelX%, levelY%).TypeId <> TILETYPEITEM THEN
        EXIT SUB
    END IF


    itemIdx% = -1
    itemFound% = FALSE
       
    'locate in item array
    FOR i% = 0 TO LevelInfo.NumItems
        IF Item(i%).levelX = levelX% AND (Item(i%).levelY = levelY% OR Item(i%).levelY = levelY%) THEN
            itemIdx% = i%
            itemFound% = TRUE
            EXIT FOR
        END IF

    NEXT i%

    IF itemFound% = FALSE THEN EXIT SUB

    IF Item(itemIdx%).IsPickedUp THEN EXIT SUB


    CALL DrawSpriteMask(Item(itemIdx%).x, Item(itemIdx%).y)
   

    CALL PlaySfx(PICKUPITEMSFX)

    Item(itemIdx%).IsPickedUp = TRUE

    SELECT CASE Item(itemIdx%).attribute


        ' increases player lives by the value of the life item.
        CASE LIFEATTR
            Player.Lives = Player.Lives + Item(itemIdx%).value
            CALL DrawStatusInfo(FALSE, TRUE, FALSE, FALSE, FALSE, FALSE)



        ' delivers an undelivered present for santa
        CASE HELPELFATTR

            CALL ClearKbBuffer
            ERASE KeyState 'reset pressed keys to off

            LINE (20, 25)-(212, 100), 12, BF
            LINE (22, 26)-(210, 98), 0, BF
             
            LOCATE 6, 9
            COLOR 15
            PRINT "Thanks Santa!"
            LOCATE 8, 5
            PRINT "I'll deliver one of"
            LOCATE 9, 5
            PRINT "your presents for you!"
            LOCATE 11, 4
            COLOR 7
            PRINT "PRESS ENTER TO CONTINUE"

            DO
            LOOP WHILE INKEY$ <> CHR$(13)

            'Hack-y way to redraw the current play field....
            CLS
            CALL DrawBackground
            CALL DrawStatusInfo(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE)
            CALL DrawLevelSpriteMap(TRUE, FALSE)
            

            FOR i% = 0 TO LevelInfo.NumItems

                IF Item(i%).IsPickedUp = FALSE THEN
                    CALL DrawSprite(Item(i%).x - 1, Item(i%).y - 1, Item(i%).SpriteId)
                END IF

            NEXT i%



            IF LevelInfo.NumPresentsDelivered < LevelInfo.NumPresents THEN
                LevelInfo.NumPresentsDelivered = LevelInfo.NumPresentsDelivered + 1
                CALL DrawStatusInfo(FALSE, FALSE, TRUE, FALSE, FALSE, FALSE)
            END IF


        'increases level timer by the value of the timer
        'item
        CASE TIMERATTR
            LevelInfo.CurTimerVal = LevelInfo.CurTimerVal + Item(itemIdx%).value
            CALL DrawStatusInfo(FALSE, FALSE, FALSE, FALSE, FALSE, FALSE)


        'puts all enemies to sleep for the value of the
        'sleep item
        CASE SLEEPATTR
            IF LevelInfo.NumEnemies > 0 THEN
                FOR i% = 0 TO LevelInfo.NumEnemies
                    IF Enemy(i%).IsRemoved = FALSE THEN
                        Enemy(i%).isActive = FALSE
                        Enemy(i%).CurSleepTicks = 0
                        Enemy(i%).CurMaxSleepTicks = Item(itemIdx%).value

                        CALL DrawSpriteMask(Enemy(i%).x, Enemy(i%).y)
                    END IF
                NEXT i%
            END IF


        'Wakes up all non-removed enemies and increases their
        'speed by the value of the alarm item
        CASE ALARMATTR
            IF LevelInfo.NumEnemies > 0 THEN
                FOR i% = 0 TO LevelInfo.NumEnemies
                    IF Enemy(i%).IsRemoved = FALSE THEN
                        Enemy(i%).isActive = TRUE
                        Enemy(i%).CurEnemyStep = Enemy(i%).CurEnemyStep + Item(itemIdx%).value
                    END IF
                NEXT i%
            END IF





    END SELECT


END SUB

SUB CheckPickUpPresent (levelX%, topLevelY%)

    IF LevelTileInfoMap(levelX%, topLevelY%).attributes = SLEDATTR AND LevelTileInfoMap(levelX%, topLevelY%).TypeId = TILETYPEMARKER THEN
          
        IF Player.HasPresent = FALSE THEN

            IF LevelInfo.NumPresentsDelivered < LevelInfo.NumPresents THEN

                Player.HasPresent = TRUE

                CALL GenPresentColors

                CALL DrawStatusInfo(FALSE, FALSE, FALSE, TRUE, FALSE, FALSE)
                CALL PlaySfx(PICKUPPRESENTSFX)

                CALL ChangePlayerHealth(HEALTHPICKUPBONUS)

                ' Player resets at the pick up now if they die.
                LevelInfo.PlayerStartX = Player.x
                LevelInfo.PlayerStartY = Player.y



            ELSE

                LevelInfo.LevelComplete = TRUE

            END IF
            
        END IF
    END IF

END SUB

SUB DrawBackground


    'Draws background boxes outside of level map


    ' level timer

    LINE (245, 11)-(305, 25), 15, B
 
    COLOR 15
    LOCATE 2, 33
    PRINT "TIMER"


    ' lives box

    LINE (245, 35)-(305, 50), 15, B
  
    COLOR 15
    LOCATE 5, 33
    PRINT "LIVES"


    ' delivery counter

    LINE (245, 59)-(305, 75), 15, B

    COLOR 15
    LOCATE 8, 32
    PRINT "DELIVER"



    ' present box
       
    LINE (245, 83)-(305, 115), 15, B
 
    COLOR 15
    LOCATE 11, 32
    PRINT "PRESENT"

   
    ' Walk mode box

    LINE (245, 123)-(305, 155), 15, B

    COLOR 15
    LOCATE 16, 33
    PRINT "MODE"
   
   
    ' item box - TODO: Moved to V1.1 due to time constraints

    'LINE (245, 123)-(305, 160), 15, B

    'COLOR 15
    'LOCATE 16, 33
    'PRINT "ITEM"



    'spirit bar
   
    COLOR 15
    LOCATE 22, 1
    PRINT "Christmas Spirit: "



END SUB

SUB DrawDebugGrid
    FOR yOffset% = 0 TO MAXLEVELHEIGHT STEP MAXSPRITEHEIGHT
        FOR xOffset% = 0 TO MAXLEVELWIDTH STEP MAXSPRITEWIDTH
  
            LINE (xOffset%, yOffset%)-(xOffset% + MAXSPRITEWIDTH, yOffset% + MAXSPRITEHEIGHT), 8, B

        NEXT xOffset%
    NEXT yOffset%

END SUB

SUB DrawEnemy (enemyIdx%)

    IF enemyIdx% < 0 OR enemyIdx% > MAXENEMIES THEN
        EXIT SUB
    END IF

    'draw enemy based on dir and animation frame


    ' Using DrawSprite instead of PUT as it allows
    ' dynamic enemy sprites instead of shared ones.

    
    IF Enemy(enemyIdx%).SpriteFrame.Direction = FORWARD THEN
        SELECT CASE Enemy(enemyIdx%).SpriteFrame.forwardFrameNum

            CASE 0
                CALL DrawSprite(Enemy(enemyIdx%).x, Enemy(enemyIdx%).y - 1, Enemy(enemyIdx%).SpriteFrame.FirstSpriteId)
                'PUT (Enemy(enemyIdx%).x, Enemy(enemyIdx%).y), EnemySpriteF0, PSET

            CASE 1
                CALL DrawSprite(Enemy(enemyIdx%).x, Enemy(enemyIdx%).y - 1, Enemy(enemyIdx%).SpriteFrame.FirstSpriteId + 1)
                'PUT (Enemy(enemyIdx%).x, Enemy(enemyIdx%).y), EnemySpriteF1, PSET

        END SELECT

        Enemy(enemyIdx%).SpriteFrame.forwardFrameNum = Enemy(enemyIdx%).SpriteFrame.forwardFrameNum + 1
        IF Enemy(enemyIdx%).SpriteFrame.forwardFrameNum > 1 THEN
            Enemy(enemyIdx%).SpriteFrame.forwardFrameNum = 0
        END IF


    ELSE

        SELECT CASE Enemy(enemyIdx%).SpriteFrame.backwardFrameNum

            CASE 0
                CALL DrawSprite(Enemy(enemyIdx%).x - 1, Enemy(enemyIdx%).y - 1, Enemy(enemyIdx%).SpriteFrame.FirstSpriteId + 2)
                'PUT (Enemy(enemyIdx%).x, Enemy(enemyIdx%).y), EnemySpriteB0, PSET

            CASE 1
                CALL DrawSprite(Enemy(enemyIdx%).x - 1, Enemy(enemyIdx%).y - 1, Enemy(enemyIdx%).SpriteFrame.FirstSpriteId + 3)
                'PUT (Enemy(enemyIdx%).x, Enemy(enemyIdx%).y), EnemySpriteB1, PSET

        END SELECT

        Enemy(enemyIdx%).SpriteFrame.backwardFrameNum = Enemy(enemyIdx%).SpriteFrame.backwardFrameNum + 1
        IF Enemy(enemyIdx%).SpriteFrame.backwardFrameNum > 1 THEN
            Enemy(enemyIdx%).SpriteFrame.backwardFrameNum = 0
        END IF


    END IF


END SUB

SUB DrawFgTiles


    ' This still isn't pretty and is stil all CPU sink but
    ' at least it's a bit faster than it was in the first demo....
    ' Tested with a bunch of water sprites in one level and didn't
    ' notice any slow down this 486 computer.

             
    Player.levelX = INT((Player.x) / MAXSPRITEWIDTH) + 1
    Player.levelY = INT((Player.y + MAXSPRITEHEIGHT) / MAXSPRITEHEIGHT) + 1


    startY% = Player.levelY - 2
    IF startY% < 0 THEN startY% = 0

    endY% = Player.levelY + 1
    IF endY% > UBOUND(LevelSpriteMap, 2) THEN endY% = UBOUND(LevelSpriteMap, 2)

    startX% = Player.levelX - 1
    IF startX% < 0 THEN startX% = 0

    endX% = Player.levelX + 1
    IF endX% > UBOUND(LevelSpriteMap, 1) THEN endX% = UBOUND(LevelSpriteMap, 1)



    FOR mapY% = startY% TO endY%
        FOR mapX% = startX% TO endX%

  
            IF LevelTileInfoMap(mapX%, mapY%).TypeId = TILETYPEMARKER THEN

                SELECT CASE LevelTileInfoMap(mapX%, mapY%).attributes

                   
                    ' Can stack CASEs to always redraw markers with those
                    ' attribute


                    CASE WATERATTR

                        'need the upper left most corner + 1
                        spriteDrawX% = (mapX% * MAXSPRITEWIDTH) - MAXSPRITEWIDTH - 1
                        spriteDrawY% = (mapY% * MAXSPRITEHEIGHT) - MAXSPRITEHEIGHT - 1

                        CALL DrawSprite(spriteDrawX%, spriteDrawY%, LevelSpriteMap(mapX%, mapY%))

                END SELECT
            END IF

        NEXT mapX%
    NEXT mapY%
   


END SUB

SUB DrawLevelSpriteMap (drawLevelOnly%, drawPlayerStart%)
   
    FOR mapY% = 0 TO UBOUND(LevelSpriteMap, 2)
        FOR mapX% = 0 TO UBOUND(LevelSpriteMap, 1)

            'need the upper left most corner + 1
            spriteDrawX% = (mapX% * MAXSPRITEWIDTH) - MAXSPRITEWIDTH - 1
            spriteDrawY% = (mapY% * MAXSPRITEHEIGHT) - MAXSPRITEHEIGHT - 1

           
            'Draw some select marker sprites.
            IF LevelTileInfoMap(mapX%, mapY%).TypeId = TILETYPEMARKER THEN
               
                SELECT CASE LevelTileInfoMap(mapX%, mapY%).attributes
                    CASE SLEDATTR, TREEATTR, WATERATTR
                        CALL DrawSprite(spriteDrawX%, spriteDrawY%, LevelSpriteMap(mapX%, mapY%))
                END SELECT

           
            ELSEIF drawLevelOnly% = TRUE AND LevelTileInfoMap(mapX%, mapY%).TypeId <> TILETYPEBLOCKING THEN
                ' do nothing
           
          
            ELSE
                CALL DrawSprite(spriteDrawX%, spriteDrawY%, LevelSpriteMap(mapX%, mapY%))
            END IF

           
            ' make sure play start position is drawn if specified.
            IF drawPlayerStart% THEN
                IF LevelTileInfoMap(mapX%, mapY%).TypeId = TILETYPEMARKER AND LevelTileInfoMap(mapX%, mapY%).attributes = PLAYERATTR THEN
                    CALL DrawSprite(spriteDrawX%, spriteDrawY%, LevelSpriteMap(mapX%, mapY%))
                END IF
            END IF

        NEXT mapX%
    NEXT mapY%

    'need FIX because constants are calculated and need to ignore
    'any value after decimal place
    LINE (MAXSPRITEWIDTH * FIX(MAXHORIZONTALSPRITES), 1)-(MAXSPRITEWIDTH * FIX(MAXHORIZONTALSPRITES), MAXSPRITEHEIGHT * FIX(MAXVERTICALSPRITES)), 15
    LINE -(1, MAXSPRITEHEIGHT * FIX(MAXVERTICALSPRITES)), 15

END SUB

SUB DrawPlayer

    IF Player.y < 0 THEN EXIT SUB

    'blending doesn't have direction or animations
    IF Player.IsBlending THEN
        'CALL DrawSprite(Player.x, Player.y - 1, Player.SpriteFrame.BlendSpriteId)
        PUT (Player.x, Player.y), PlayerSpriteBlend, PSET
        EXIT SUB
    END IF


    'draw player based on dir and animation frame
    IF Player.SpriteFrame.Direction = FORWARD THEN
        SELECT CASE Player.SpriteFrame.forwardFrameNum

            CASE 0
                'CALL DrawSprite(Player.x, Player.y - 1, Player.SpriteFrame.FirstSpriteId)
                PUT (Player.x, Player.y), PlayerSpriteF0, PSET

            CASE 1
                'CALL DrawSprite(Player.x, Player.y - 1, Player.SpriteFrame.FirstSpriteId + 1)
                PUT (Player.x, Player.y), PlayerSpriteF1, PSET

        END SELECT

        Player.SpriteFrame.forwardFrameNum = Player.SpriteFrame.forwardFrameNum + 1
        IF Player.SpriteFrame.forwardFrameNum > 1 THEN
            Player.SpriteFrame.forwardFrameNum = 0
        END IF


    ELSE

        SELECT CASE Player.SpriteFrame.backwardFrameNum

            CASE 0
                'CALL DrawSprite(Player.x, Player.y - 1, Player.SpriteFrame.FirstSpriteId + 2)
                PUT (Player.x, Player.y), PlayerSpriteB0, PSET

            CASE 1
                'CALL DrawSprite(Player.x, Player.y - 1, Player.SpriteFrame.FirstSpriteId + 3)
                PUT (Player.x, Player.y), PlayerSpriteB1, PSET

        END SELECT

        Player.SpriteFrame.backwardFrameNum = Player.SpriteFrame.backwardFrameNum + 1
        IF Player.SpriteFrame.backwardFrameNum > 1 THEN
            Player.SpriteFrame.backwardFrameNum = 0
        END IF


    END IF



END SUB

SUB DrawSprite (xOffset%, yOffset%, SpriteId%)

    FOR y% = 1 TO MAXSPRITEHEIGHT
        FOR x% = 1 TO MAXSPRITEWIDTH

            'skips 'PSET'ing on blank sprite data to save draw time.

            IF SpriteMap(SpriteId%, x%, y%) > 0 THEN
               
                'PSET (x% + xOffset%, y% + yOffset%), SpriteMap(SpriteId%, x%, y%)
             
                ' Much faster than PSET directly!
                DEF SEG = &HA000
                    POKE (x% + xOffset%) + ((y% + yOffset%) * 320&), SpriteMap(SpriteId%, x%, y%)
                DEF SEG

            END IF

        NEXT x%
    NEXT y%


END SUB

SUB DrawSpriteMask (xOffset%, yOffset%)

   
    'Draws black box the size of a sprite at the x,y supplied.

    IF xOffset% < 0 OR yOffset% < 0 THEN EXIT SUB

    PUT (xOffset%, yOffset%), MaskSprite, AND
    

END SUB

SUB DrawStatusInfo (redrawSpiritBar%, redrawLives%, redrawDeliverCount%, redrawPresent%, redrawItem%, redrawMode%)


    ' Containers for status items are drawn in the
    ' DrawBackground SUB at start of level to decrease the
    ' amount of needless screen redrawing.




    ' level timer

    COLOR 15
    timerXPos% = 33

    IF LevelInfo.CurTimerVal < 10 THEN
        COLOR 4
        timerXPos% = 34
    ELSEIF LevelInfo.CurTimerVal < 100 THEN
        COLOR 14
        timerXPos% = 34
    END IF

    IF LevelInfo.CurTimerVal > 0 THEN
        LOCATE 3, timerXPos%
        PRINT LevelInfo.CurTimerVal
    ELSE
        LOCATE 3, timerXPos%
        PRINT " -  "
    END IF



    ' lives box

    IF redrawLives% THEN

        livesXPos% = 34
        

        IF Player.Lives >= 0 THEN
            LOCATE 6, livesXPos%
            PRINT Player.Lives
        ELSE
            LOCATE 6, livesXPos% + 1
            PRINT "-"
        END IF

    END IF



    ' delivered presents count

    IF redrawDeliverCount% THEN
        LOCATE 9, 32
        PRINT LevelInfo.NumPresentsDelivered; "/"; LevelInfo.NumPresents
    END IF



    ' present box

    IF redrawPresent% THEN

        
        IF Player.HasPresent THEN
            LINE (260, 95)-(285, 112), LevelInfo.CurPresentBgColor, BF
            LINE (270, 95)-(275, 112), LevelInfo.CurPresentFgColor, BF
            LINE (260, 102)-(285, 106), LevelInfo.CurPresentFgColor, BF
            PSET (272, 95), LevelInfo.CurPresentFgColor
            DRAW "u1 l1 u1 l1 u1 l1 u1 l1 u1 l1 u1 l3 d1 l1 d1 l1 r1 d1 r2 d1 r3 d1 r2"
            PSET (272, 95), LevelInfo.CurPresentFgColor
            DRAW "u1 r1 u1 r1 u1 r1 u1 r1 u1 r1 u1 r3 d1 r1 d1 r1 l1 d1 l2 d1 l3 d1 l2"

        ELSE
            LINE (255, 87)-(295, 114), 0, BF
        END IF

    END IF

    ' walk mode

    IF redrawMode% THEN
   
        IF Player.IsBlending THEN
            LOCATE 18, 33
            PRINT "BLEND"
        ELSEIF Player.IsRunning THEN
            LOCATE 18, 33
            PRINT "RUN  "
        ELSE
            LOCATE 18, 33
            PRINT "WALK "
    
        END IF
    
    END IF



    ' item box - TODO: Moved to V1.1 due to time constraints!

    'IF redrawItem% THEN
    '
    '    ' TODO : use sprite here
    '    IF Player.hasItem THEN
    '        LINE (255, 130)-(295, 155), 2, BF
    '    ELSE
    '        LINE (255, 130)-(295, 155), 0, BF
    '
    '    END IF
    '
    'END IF





    'spirit bar

    IF redrawSpiritBar% THEN

        spiritColor% = 15

        IF Player.health < 25 THEN
            spiritColor% = 4
        ELSEIF Player.health < 50 THEN
            spiritColor% = 14
        ELSEIF Player.health < 75 THEN
            spiritColor% = 10
        END IF

        LINE (1, 180)-(200, 185), 0, BF
  
        healthBarValue% = Player.health * 2
        IF healthBarValue% > 1 THEN

            LINE (1, 180)-(healthBarValue%, 185), spiritColor%, BF

        END IF
    END IF


END SUB

SUB EndOfDemo

    CALL ClearKbBuffer

    LINE (20, 25)-(272, 95), 12, BF
    LINE (22, 26)-(270, 93), 0, BF
             
    LOCATE 6, 7
    COLOR 15
    PRINT "CONGRATS! DEMO COMPLETE!"
    LOCATE 8, 4
    COLOR 7
    PRINT " That's it for now! Check back"
    LOCATE 9, 4
    PRINT " back later for the full game!"
    LOCATE 11, 7
    COLOR 8
    PRINT "PRESS ENTER TO CONTINUE"


    DO
    LOOP WHILE INKEY$ <> CHR$(13)


END SUB

SUB ExitScreen


    SCREEN 0
    WIDTH 80
    CLS
    COLOR 4, 2
    PRINT "HAPPY "
    COLOR 2, 4
    LOCATE 1, 7
    PRINT " HOLIDAYS!"
    COLOR 15, 0
    PRINT "2022 Nukem Enterprises"
    PRINT "http://eriks.servehttp.com"
    PRINT
    COLOR 7, 0
    PRINT
    PRINT "Thanks for playing! This is just the demo of the game as "
    PRINT "I ran out of time to finish everything before Christmas. "
    PRINT "Hopefully you enjoyed the demo! The full version with "
    PRINT "more levels and features will be out soon!"
    PRINT
    PRINT


END SUB

SUB GenPresentColors

        'Randomly set the present's color scheme each pick up.
   
        LevelInfo.CurPresentFgColor = INT(15 * RND) + 1
        LevelInfo.CurPresentBgColor = INT(15 * RND) + 1

        IF LevelInfo.CurPresentFgColor = LevelInfo.CurPresentBgColor THEN
            LevelInfo.CurPresentFgColor = LevelInfo.CurPresentFgColor + 1

            IF LevelInfo.CurPresentFgColor > 15 THEN
                LevelInfo.CurPresentFgColor = 1
            END IF
        END IF
               

END SUB

SUB HandleDeath

    Player.Lives = Player.Lives - 1
    Player.health = MAXHEALTH

    CALL DrawSpriteMask(Player.x, Player.y)

    Player.x = LevelInfo.PlayerStartX
    Player.y = LevelInfo.PlayerStartY
    Player.IsBlending = FALSE
    Player.hasItem = FALSE
    Player.HasPresent = FALSE

    'reset timer
    LevelInfo.CurTimerVal = LevelInfo.StartTimerVal

    IF Player.Lives < 0 THEN
        Player.health = 0
        LevelInfo.CurTimerVal = 0
        Player.IsGameOver = TRUE


    ELSE
        CALL DrawPlayer
        CALL PlaySfx(PLAYERDIEDSFX)
    END IF

   
    CALL DrawStatusInfo(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE)


END SUB

SUB HandleEnemyAttack (enemyIdx%)


    enemyAttacked% = FALSE

    IF Player.HasPresent THEN
       
        ' Give present to enemy to go away for good.
        ' If no presents were delivered, it's game over.

       
        Player.HasPresent = FALSE

        CALL ChangePlayerHealth(-20)

        Enemy(enemyIdx%).IsRemoved = TRUE

        LevelInfo.NumPresents = LevelInfo.NumPresents - 1

        IF LevelInfo.NumPresents <= 0 THEN
            Player.IsGameOver = TRUE
        END IF

   
        enemyAttacked% = TRUE
        
   
    'TODO: ELSEIFs for when player has potential items to ward off enemies

   
    ' If no present or item, hurt bad and sleep short
    ELSE
        CALL ChangePlayerHealth(-40)
        Enemy(enemyIdx%).CurMaxSleepTicks = Enemy(enemyIdx%).MaxSleepTicks / 2

        enemyAttacked% = TRUE

    END IF


    IF enemyAttacked% THEN

        CALL PlaySfx(PLAYERHURTSFX)

        Enemy(enemyIdx%).isActive = FALSE
        Enemy(enemyIdx%).CurSleepTicks = 0

        CALL DrawSpriteMask(Enemy(enemyIdx%).x, Enemy(enemyIdx%).y)

        CALL DrawStatusInfo(FALSE, FALSE, TRUE, TRUE, TRUE, FALSE)
    END IF

END SUB

SUB HandleHighScore

    ' Checks if player's score is a high score. If so, it will
    ' save it to the high score data file in the appropriate spot.

   
    hdataExists% = CheckFileExists(HIGHSCOREFILE)

  
    'If doesn't exist, create a new file

    IF hdataExists% = FALSE THEN
        f% = FREEFILE
        OPEN HIGHSCOREFILE FOR APPEND AS #f%
        CLOSE #f%
    END IF

  
    REDIM HighScore(MAXHIGHSCORES) AS HighScoreData
 

    ' TODO : FINISH THIS! Just started... doesn't work yet!!!

    playerRecIdx% = -1
  
    hId% = FREEFILE
    OPEN HIGHSCOREFILE FOR INPUT AS #hId%

        curRec% = 0
        playerRecSet% = FALSE

        WHILE NOT EOF(hId%) AND curRec% <= UBOUND(HighScore)
            INPUT #hId%, HighScore(curRec%).DateStr, HighScore(curRec%).PlayerName, HighScore(curRec%).Score


            IF Player.Score >= HighScore(curRec%).Score AND playerRecSet% = FALSE THEN
                playerRecIdx% = curRec%
                playerRecSet% = TRUE
            END IF

            curRec% = curRec% + 1
        WEND

        IF curRec% = 0 THEN playerRecIdx% = 0


    CLOSE #hId%



    IF playerRecIdx% >= LBOUND(HighScore) AND playerRecIdx% <= UBOUND(HighScore) THEN

        name$ = ""
        WHILE name$ = ""
       
            CLS

            FOR i% = 0 TO 150

                snowX% = RND * 320 + 1
                snowY% = RND * 200 + 1

                PSET (snowX%, snowY%), 15

            NEXT i%

            COLOR 10
            LOCATE 1, 14
            PRINT "NEW HIGH SCORE!"
            LOCATE 2, 8
            PRINT ""
            PRINT
            COLOR 7

            LOCATE 3, 5
            INPUT "Enter your name: ", name$
        WEND


        FOR i% = UBOUND(HighScore) TO playerRecIdx% - 1 STEP -1

            nextRec% = i% + 1
            IF nextRec% <= UBOUND(HighScore) AND i% >= 0 THEN
                HighScore(nextRec%).DateStr = HighScore(i%).DateStr
                HighScore(nextRec%).PlayerName = HighScore(i%).PlayerName
                HighScore(nextRec%).Score = HighScore(i%).Score
            END IF


        NEXT i%

        HighScore(playerRecIdx%).DateStr = DATE$
        HighScore(playerRecIdx%).PlayerName = name$
        HighScore(playerRecIdx%).Score = Player.Score
        


        hOutId% = FREEFILE
        OPEN HIGHSCOREFILE FOR OUTPUT AS #hOutId%

            

            FOR i% = 0 TO UBOUND(HighScore)
                WRITE #hOutId%, HighScore(i%).DateStr, HighScore(i%).PlayerName, HighScore(i%).Score

            NEXT i%


        CLOSE #hOutId%


    END IF




    CLS
 
    FOR i% = 0 TO 150

        snowX% = RND * 320 + 1
        snowY% = RND * 200 + 1

        PSET (snowX%, snowY%), 15

    NEXT i%

    COLOR 10
    LOCATE 1, 14
    PRINT "HIGH SCORES"
    LOCATE 2, 13
    PRINT ""
    PRINT
    COLOR 7

    FOR i% = 0 TO UBOUND(HighScore)
        IF HighScore(i%).Score <> 0 THEN
            LOCATE 3 + i%, 2
            PRINT HighScore(i%).DateStr; " "; HighScore(i%).PlayerName; HighScore(i%).Score
        END IF

    NEXT i%


    LOCATE 23, 9
    COLOR 15
    PRINT "PRESS ENTER TO RETURN"
    DO: LOOP WHILE INKEY$ <> CHR$(13)




END SUB

SUB HandleLevelComplete


    CALL PlaySfx(LEVELCOMPLETESFX)


    LevelInfo.LevelNum = LevelInfo.LevelNum + 1


    CALL ClearKbBuffer


    presentScore% = LevelInfo.NumPresentsDelivered * DELIVEREDPRESENTSCORE
    timerScore% = LevelInfo.CurTimerVal * TIMESCORE
    lifeScore% = Player.Lives * LIVESSCORE
    totalScore% = presentScore% + timerScore% + lifeScore%
    Player.Score = Player.Score + totalScore%



    LINE (20, 25)-(232, 140), 12, BF
    LINE (22, 26)-(230, 138), 0, BF
             
    LOCATE 6, 9
    COLOR 15
    PRINT "LEVEL COMPLETE!"

    LOCATE 8, 5
    PRINT "    TIME x 0"; LTRIM$(STR$(TIMESCORE));
    LOCATE 8, 22
    PRINT "="; timerScore%
   
    LOCATE 9, 5
    PRINT "PRESENTS x "; LTRIM$(STR$(LevelInfo.NumPresentsDelivered));
    LOCATE 9, 22
    PRINT "="; presentScore%
   
    LOCATE 10, 5
    PRINT "   LIVES x "; LTRIM$(STR$(Player.Lives));
    LOCATE 10, 22
    PRINT "="; lifeScore%
   
   
    LOCATE 11, 8
    PRINT "LEVEL TOTAL   ="; totalScore%
    LOCATE 12, 5
    PRINT ""
    LOCATE 14, 7
    PRINT "TOTAL SCORE:"; Player.Score



    LOCATE 16, 5
    COLOR 7
    PRINT "PRESS ENTER TO CONTINUE"


    DO
    LOOP WHILE INKEY$ <> CHR$(13)

END SUB

SUB HandleLvlTimer

    LevelInfo.CurTimerVal = LevelInfo.CurTimerVal - 1

  
    CALL DrawStatusInfo(FALSE, FALSE, FALSE, FALSE, FALSE, FALSE)



    IF LevelInfo.CurTimerVal < 0 THEN

        CALL HandleDeath

    END IF



END SUB

SUB HighScores


    hdataExists% = CheckFileExists(HIGHSCOREFILE)

   
    'If doesn't exist, create a new file

    IF hdataExists% = FALSE THEN
        f% = FREEFILE
        OPEN HIGHSCOREFILE FOR APPEND AS #f%
        CLOSE #f%
    END IF

   
    REDIM HighScore(MAXHIGHSCORES) AS HighScoreData
   
   
    hId% = FREEFILE
    OPEN HIGHSCOREFILE FOR INPUT AS #hId%

        curRec% = 0

        WHILE NOT EOF(hId%) AND curRec% <= UBOUND(HighScore)
            INPUT #hId%, HighScore(curRec%).DateStr, HighScore(curRec%).PlayerName, HighScore(curRec%).Score
            curRec% = curRec% + 1
            
        WEND


    CLOSE #hId%


    CLS
  
    FOR i% = 0 TO 150

        snowX% = RND * 320 + 1
        snowY% = RND * 200 + 1

        PSET (snowX%, snowY%), 15

    NEXT i%

    COLOR 10
    LOCATE 1, 14
    PRINT "HIGH SCORES"
    LOCATE 2, 13
    PRINT ""
    PRINT
    COLOR 7

    FOR i% = 0 TO UBOUND(HighScore)
        IF HighScore(i%).Score <> 0 THEN
            LOCATE 4 + i%, 2
            PRINT HighScore(i%).DateStr; " "; HighScore(i%).PlayerName; HighScore(i%).Score
        END IF

    NEXT i%


    LOCATE 23, 9
    COLOR 15
    PRINT "PRESS ENTER TO RETURN"
    DO: LOOP WHILE INKEY$ <> CHR$(13)


END SUB

SUB Init (LevelFileName$)

    'set up mask sprite
    CLS
  
    FOR y% = 0 TO MAXSPRITEHEIGHT - 1
        FOR x% = 0 TO MAXPSRITEWIDTH - 1
            PSET (x%, y%), 0
        NEXT x%
    NEXT y%

    GET (0, 0)-((MAXSPRITEWIDTH) - 1, MAXSPRITEHEIGHT - 1), MaskSprite

    CLS

    'reset enemy array. Will be repopluated from LoadLevel SUB
    FOR i% = 0 TO UBOUND(Enemy, 1)
        Enemy(i%).x = 0
        Enemy(i%).y = 0
        Enemy(i%).MaxSleepTicks = DEFAULTENEMYSLEEPDURATION
        Enemy(i%).CurMaxSleepTicks = DEFAULTENEMYSLEEPDURATION
        Enemy(i%).CurSleepTicks = 0
        Enemy(i%).isActive = FALSE
        Enemy(i%).IsRemoved = TRUE
        Enemy(i%).startX = 0
        Enemy(i%).startY = 0
        Enemy(i%).CurEnemyStep = DEFAULTENEMYSTEP
        Enemy(i%).SpriteFrame.FirstSpriteId = 0
        Enemy(i%).SpriteFrame.BlendSpriteId = 0
        Enemy(i%).SpriteFrame.Direction = FORWARD
        Enemy(i%).SpriteFrame.forwardFrameNum = 0
        Enemy(i%).SpriteFrame.backwardFrameNum = 0

    NEXT i%

    'reset item array. Will be repopluated from LoadLevel SUB
    FOR i% = 0 TO UBOUND(Item, 1)
        Item(i%).levelX = 0
        Item(i%).levelY = 0
        Item(i%).x = 0
        Item(i%).y = 0
        Item(i%).value = 0
        Item(i%).IsPickedUp = FALSE
        Item(i%).SpriteId = 0
        Item(i%).attribute = ""
    NEXT i%


    LevelInfo.NumEnemies = 0
    LevelInfo.NumItems = 0

    'reset player values to start values
    Player.StepSize = DEFAULTPLAYERSTEP

    Player.SpriteFrame.Direction = FORWARD
    Player.health = MAXHEALTH
    Player.IsBlending = FALSE
    Player.IsRunning = FALSE
    Player.IsJumping = FALSE
    Player.IsGameOver = FALSE
    
    Player.HasPresent = TRUE
    Player.hasItem = FALSE

   
    CALL LoadLevel(LevelFileName$)



    Player.levelX = INT((Player.x) / MAXSPRITEWIDTH) + 1
    Player.levelY = INT((Player.y + MAXSPRITEHEIGHT) / MAXSPRITEHEIGHT) + 1


    'Set additional level info values based on loaded values.
    LevelInfo.PlayerStartX = Player.x
    LevelInfo.PlayerStartY = Player.y

  
    LevelInfo.CurTimerVal = LevelInfo.StartTimerVal

    LevelInfo.NumPresentsDelivered = 0
    LevelInfo.LevelComplete = FALSE


    CALL DrawBackground
    CALL GenPresentColors
    CALL DrawStatusInfo(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE)



END SUB

SUB Instructions

    

    CLS
   
    FOR i% = 0 TO 150

        snowX% = RND * 320 + 1
        snowY% = RND * 200 + 1

        PSET (snowX%, snowY%), 15

    NEXT i%

    COLOR 10
    LOCATE 1, 9
    PRINT "SANTA'S DELIVERY RUSH"
    LOCATE 2, 8
    PRINT ""
    PRINT
    COLOR 7
    PRINT "  Oh no! It's Christmas Eve and Santa"
    PRINT "has over slept! Time is ticking so he"
    PRINT "will need to be as fast as fast can be"
    PRINT "in order to deliver all the presents"
    PRINT "on time."
    PRINT
    PRINT "In this game your goal is to deliver"
    PRINT "all the presents in the level before"
    PRINT "the timer runs out. Beware of the the"
    PRINT "Christmas snoopers trying to catch "
    PRINT "Santa in the act!"
    PRINT
    PRINT "If you have a present you can give it"
    PRINT "to them and they will go back to bed."
    PRINT "If you don't they'll only rest for a"
    PRINT "bit before coming back out to find you!"

    LOCATE 23, 8
    COLOR 15
    PRINT "PRESS ENTER TO CONTINUE"
    DO: LOOP WHILE INKEY$ <> CHR$(13)

   
    CLS
 
    FOR i% = 0 TO 150

        snowX% = RND * 320 + 1
        snowY% = RND * 200 + 1

        PSET (snowX%, snowY%), 15

    NEXT i%

    COLOR 10
    LOCATE 1, 16
    PRINT "CONTROLS"
    LOCATE 2, 15
    PRINT ""

    
    COLOR 7
    LOCATE 4, 8
    PRINT "ARROW KEYS - Move"
    LOCATE 5, 9
    PRINT "SPACE BAR - Jump"
    LOCATE 6, 17
    PRINT "B - Blend On/Off"
    LOCATE 7, 17
    PRINT "Z - Run On/Off"
    LOCATE 8, 17
    PRINT "S - Sound On/Off"
    LOCATE 9, 17
    PRINT "Q - Quit"

    LOCATE 11, 1
    PRINT "When blended Santa will move at half"
    PRINT "speed but will not be seen by snoopers."
    PRINT "This will also drain his Christmas"
    PRINT "Spirit."
    PRINT
    PRINT "When running, Santa will move at double"
    PRINT "speed but will drain Christmas Spirit"
    PRINT "until he returns to Walk mode."

    LOCATE 23, 9
    COLOR 15
    PRINT "PRESS ENTER TO CONTINUE"
    DO: LOOP WHILE INKEY$ <> CHR$(13)

   
   
    CLS
  
    FOR i% = 0 TO 150

        snowX% = RND * 320 + 1
        snowY% = RND * 200 + 1

        PSET (snowX%, snowY%), 15

    NEXT i%

    COLOR 10
    LOCATE 1, 17
    PRINT "ITEMS"
    LOCATE 2, 16
    PRINT ""

   
    ' Relies on these being in the right SID.

    CALL DrawSprite(30, 25, 7)
    CALL DrawSprite(30, 48, 8)
    CALL DrawSprite(30, 72, 9)
    CALL DrawSprite(30, 97, 17)
    CALL DrawSprite(30, 122, 18)
   
    COLOR 7
    LOCATE 5, 7
    PRINT " - Add extra life"
    LOCATE 8, 7
    PRINT " - Add time to the level timer"
    LOCATE 11, 7
    PRINT " - Delivers a present for you"
    LOCATE 14, 7
    PRINT " - Wakes all enemies up"
    LOCATE 17, 7
    PRINT " - Put all enemies to sleep"
    

    
    LOCATE 23, 9
    COLOR 15
    PRINT "PRESS ENTER TO CONTINUE"
    DO: LOOP WHILE INKEY$ <> CHR$(13)



END SUB

FUNCTION IsBlockingMarkerTile% (levelX%, levelY%)

    'Checks level grid x/y passed to see if it's a marker
    'tile type that should block movement into that tile space.
   
   
    IsBlockingMarkerTile% = FALSE

    IF LevelTileInfoMap(levelX%, levelY%).TypeId = TILETYPEMARKER THEN
        SELECT CASE LevelTileInfoMap(levelX%, levelY%).attributes
           
            'list of attributes to block
            CASE SLEDATTR, TREEATTR
                IsBlockingMarkerTile% = TRUE
                    
        END SELECT

    END IF


END FUNCTION

SUB LoadLevel (LevelFileName$)
    OPEN LevelFileName$ FOR INPUT AS #1
        FOR y% = 1 TO UBOUND(LevelSpriteMap, 2)
            FOR x% = 1 TO UBOUND(LevelSpriteMap, 1)
                INPUT #1, LevelSpriteMap(x%, y%)
            NEXT x%
        NEXT y%

        FOR y% = 1 TO UBOUND(LevelTileInfoMap, 2)
            FOR x% = 1 TO UBOUND(LevelTileInfoMap, 1)
                INPUT #1, LevelTileInfoMap(x%, y%).TypeId, LevelTileInfoMap(x%, y%).value, LevelTileInfoMap(x%, y%).attributes


                IF UCASE$(LevelTileInfoMap(x%, y%).attributes) = PLAYERATTR AND LevelTileInfoMap(x%, y%).TypeId = TILETYPEMARKER THEN
                    Player.x = x% * MAXSPRITEWIDTH - MAXSPRITEWIDTH
                    Player.y = y% * MAXSPRITEHEIGHT - MAXSPRITEHEIGHT

                    'Value of sprite should be sprite id of first frame.
                    Player.SpriteFrame.FirstSpriteId = LevelTileInfoMap(x%, y%).value
                  
                    'next four sprites are hardcoded as player animations.
                    CALL SetSpriteForPut(Player.SpriteFrame.FirstSpriteId, PlayerSpriteF0())
                    CALL SetSpriteForPut(Player.SpriteFrame.FirstSpriteId + 1, PlayerSpriteF1())
                    CALL SetSpriteForPut(Player.SpriteFrame.FirstSpriteId + 2, PlayerSpriteB0())
                    CALL SetSpriteForPut(Player.SpriteFrame.FirstSpriteId + 3, PlayerSpriteB1())

                    'fifth one is the blend sprite
                    Player.SpriteFrame.BlendSpriteId = Player.SpriteFrame.FirstSpriteId + 4
                    CALL SetSpriteForPut(Player.SpriteFrame.BlendSpriteId, PlayerSpriteBlend())
                    


                ELSEIF UCASE$(LevelTileInfoMap(x%, y%).attributes) = SLEDATTR AND LevelTileInfoMap(x%, y%).TypeId = TILETYPEMARKER THEN

                    LevelInfo.NumPresents = LevelTileInfoMap(x%, y%).value

               

                ELSEIF UCASE$(LevelTileInfoMap(x%, y%).attributes) = TREEATTR AND LevelTileInfoMap(x%, y%).TypeId = TILETYPEMARKER THEN

                    LevelInfo.StartTimerVal = LevelTileInfoMap(x%, y%).value


                ELSEIF LevelTileInfoMap(x%, y%).TypeId = TILETYPEENEMY AND LevelInfo.NumEnemies < MAXENEMIES THEN

                    LevelInfo.NumEnemies = LevelInfo.NumEnemies + 1
                   
                    Enemy(LevelInfo.NumEnemies).startX = x% * MAXSPRITEWIDTH - MAXSPRITEWIDTH
                    Enemy(LevelInfo.NumEnemies).x = x% * MAXSPRITEWIDTH - MAXSPRITEWIDTH
                    Enemy(LevelInfo.NumEnemies).startY = y% * MAXSPRITEWIDTH - MAXSPRITEWIDTH
                    Enemy(LevelInfo.NumEnemies).y = y% * MAXSPRITEWIDTH - MAXSPRITEWIDTH
                    Enemy(LevelInfo.NumEnemies).isActive = TRUE

                    ' attributes is enemy sleep duration.
                    enemySleepDuration% = VAL(LevelTileInfoMap(x%, y%).attributes)
                    IF enemySleepDuration% = 0 THEN enemySleepDuration% = DEFAULTENEMYSLEEPDURATION

                    Enemy(LevelInfo.NumEnemies).MaxSleepTicks = enemySleepDuration%
                    Enemy(LevelInfo.NumEnemies).CurMaxSleepTicks = enemySleepDuration%
                                            
                   
                    'Sprite value is the start sprite id
                    Enemy(LevelInfo.NumEnemies).SpriteFrame.FirstSpriteId = LevelTileInfoMap(x%, y%).value

                    Enemy(LevelInfo.NumEnemies).IsRemoved = FALSE


                    ' Using DrawSprite instead so enemies don't share same sprite
                    'CALL SetSpriteForPut(Enemy(LevelInfo.NumEnemies).SpriteFrame.firstSpriteId, EnemySpriteF0())
                    'CALL SetSpriteForPut(Enemy(LevelInfo.NumEnemies).SpriteFrame.firstSpriteId + 1, EnemySpriteF1())
                    'CALL SetSpriteForPut(Enemy(LevelInfo.NumEnemies).SpriteFrame.firstSpriteId + 2, EnemySpriteB0())
                    'CALL SetSpriteForPut(Enemy(LevelInfo.NumEnemies).SpriteFrame.firstSpriteId + 3, EnemySpriteB1())

                    
                   
                'Populate items
                ELSEIF LevelTileInfoMap(x%, y%).TypeId = TILETYPEITEM AND LevelInfo.NumItems < MAXITEMS THEN

                    LevelInfo.NumItems = LevelInfo.NumItems + 1

                    Item(LevelInfo.NumItems).levelX = x%
                    Item(LevelInfo.NumItems).levelY = y%
                    Item(LevelInfo.NumItems).x = x% * MAXSPRITEWIDTH - MAXSPRITEWIDTH
                    Item(LevelInfo.NumItems).y = y% * MAXSPRITEWIDTH - MAXSPRITEWIDTH
                    Item(LevelInfo.NumItems).value = LevelTileInfoMap(x%, y%).value
                    Item(LevelInfo.NumItems).IsPickedUp = FALSE
                    Item(LevelInfo.NumItems).attribute = LevelTileInfoMap(x%, y%).attributes

                    Item(LevelInfo.NumItems).SpriteId = LevelSpriteMap(x%, y%)

                END IF



            NEXT x%
        NEXT y%
    CLOSE #1
    
    CALL DrawLevelSpriteMap(FALSE, TRUE)

    'LOCATE 23, 1
    'PRINT "Loaded level: "; levelFileName$


END SUB

SUB LoadSpriteIdFile
   
    fileExists% = CheckFileExists%(SPRITEIDFILE)

    IF fileExists% THEN

        OPEN SPRITEIDFILE FOR INPUT AS #1

            WHILE NOT EOF(1)
    
                INPUT #1, tempId%, tempFileName$

                IF tempId% < MAXSPRITES THEN
                    SpriteIdFileNameMap(tempId%) = tempFileName$

                    IF LTRIM$(RTRIM$(tempFileName$)) <> "" THEN
                        CALL LoadSpriteIntoMap(tempId%, tempFileName$, SpriteMap())
                    END IF

                END IF
            WEND
        CLOSE #1

        LOCATE 22, 2
        COLOR 15
        PRINT "Loaded sprite id file: "; SPRITEIDFILE

    ELSE
        LOCATE 22, 2
        COLOR 15
        PRINT SPRITEIDFILE; " Doesn't exist to load!"

    END IF


END SUB

FUNCTION MainMenu% (prevMenuChoice%)

    'This is an odd function (read:HACK) as it draws the menu and
    'gets the user input instead of just the user input.

   
    'HACK : makes the screen repaint less noticable when user
    '       is toggling sound on/off
    '
    IF prevMenuChoice% <> 3 THEN

        CLS


        FOR i% = 0 TO 150

            snowX% = RND * 320 + 1
            snowY% = RND * 200 + 1

            PSET (snowX%, snowY%), 15

        NEXT i%
    END IF

    'S
    PSET (10, 10), 4
    DRAW "r25 d8 l20 d8 r20 d20 l25 u6 r20 u8 l20 u22"
    PAINT (11, 11), 4

    'A
    PSET (45, 10), 15
    DRAW "r25 d36 l8 u16 l9 d16 l8 u36"
    PSET (53, 15), 15
    DRAW "r9 d10 l9 u10"
    PAINT (56, 11), 15

    'N
    PSET (80, 10), 4
    DRAW "r8"
    LINE (88, 10)-(102, 40), 4
    LINE -STEP(0, -30), 4
    LINE -STEP(8, 0), 4
    LINE -STEP(0, 36), 4
    LINE -STEP(-10, 0), 4
    LINE -STEP(-12, -26), 4
    LINE -STEP(0, 26), 4
    LINE -STEP(-8, 0), 4
    LINE -STEP(0, -36), 4
    PAINT (81, 11), 4

    'T
    PSET (118, 10), 15
    DRAW "r28 d8 l10 d28 l8 u28 l10 u8"
    PAINT (119, 11), 15


    'A
    PSET (154, 10), 4
    DRAW "r25 d36 l8 u16 l9 d16 l8 u36"
    PSET (162, 15), 4
    DRAW "r9 d10 l9 u10"
    PAINT (155, 11), 4

    ' '
    PSET (188, 10), 15
    DRAW "r8 d8 l8 u8"
    PAINT (189, 11), 15


    'S
    PSET (204, 10), 4
    DRAW "r25 d8 l20 d8 r20 d20 l25 u6 r20 u8 l20 u22"
    PAINT (205, 11), 4


    COLOR 10
    LOCATE 10, 22
    PRINT "DELIVERY RUSH"


    ' NOTE : This relies on sprites being in expected SID location.
    '        Levels are dynamic, this unfortunately is not.

    CALL DrawSprite(200, 90, 2)
    CALL DrawSprite(216, 90, 3)
    CALL DrawSprite(248, 90, 12)
    CALL DrawSprite(250, 20, 17)

    LOCATE 22, 15
    COLOR 7
    PRINT "V"; VERSION

    LOCATE 23, 9
    COLOR 8
    PRINT "2022 Nukem Enterprises"

    DIM menuChoice(4) AS STRING
    menuChoice(0) = "New Game"
    menuChoice(1) = "Instructions"
    menuChoice(2) = "High Scores"
   
    IF SoundOn THEN
        menuChoice(3) = "Sound: ON "
    ELSE
        menuChoice(3) = "Sound: OFF"
    END IF

    menuChoice(4) = "Exit"


    selected% = FALSE
    selectedValue% = prevMenuChoice%


    GOSUB DrawMenuSelection


    WHILE selected% = FALSE

        in$ = INKEY$

        SELECT CASE in$

           
            'Down arrow
            CASE CHR$(0) + CHR$(80)
                selectedValue% = selectedValue% + 1
                IF selectedValue% > UBOUND(menuChoice) THEN
                    selectedValue% = 0
                END IF
                GOSUB DrawMenuSelection


            'Up Arrow
            CASE CHR$(0) + CHR$(72)
                selectedValue% = selectedValue% - 1
                IF selectedValue% < 0 THEN
                    selectedValue% = UBOUND(menuChoice)
                END IF
                GOSUB DrawMenuSelection


            'Enter
            CASE CHR$(13)
                selected% = TRUE

        END SELECT


    WEND

    MainMenu% = selectedValue%


    EXIT FUNCTION


DrawMenuSelection:
    printY% = 13
    printX% = 6
    FOR i% = 0 TO UBOUND(menuChoice)
        LOCATE printY% + i%, printX%
        IF i% = selectedValue% THEN
            COLOR 15
        ELSE
            COLOR 8
        END IF
        PRINT menuChoice(i%)
    NEXT i%
    RETURN

END FUNCTION

SUB MoveEnemies

    IF LevelInfo.NumEnemies = 0 THEN EXIT SUB

    FOR i% = 0 TO LevelInfo.NumEnemies

        IF Enemy(i%).isActive AND Enemy(i%).IsRemoved = FALSE THEN

            deltaX% = 0
            enemyStep% = Enemy(i%).CurEnemyStep

            'move quicker speed at player if same level and not sneaking.
            'but only if back is not turned to player

            IF Player.IsBlending = FALSE AND Enemy(i%).y = Player.y THEN
                
                IF Player.x >= Enemy(i%).x AND Enemy(i%).SpriteFrame.Direction = FORWARD THEN
                    enemyStep% = enemyStep% + 3
                ELSEIF Player.x < Enemy(i%).x AND Enemy(i%).SpriteFrame.Direction = BACKWARD THEN
                    enemyStep% = enemyStep% + 3
                END IF
            END IF

            IF Enemy(i%).SpriteFrame.Direction = FORWARD THEN
                deltaX% = enemyStep%
            ELSE
                deltaX% = -enemyStep%
            END IF


            isCollision% = CheckEnemyCollisions(i%, deltaX%)


            IF isCollision% THEN

                'turn them around and walk the other way


                IF Enemy(i%).SpriteFrame.Direction = FORWARD THEN
                    Enemy(i%).SpriteFrame.Direction = BACKWARD
                ELSE
                    Enemy(i%).SpriteFrame.Direction = FORWARD
                END IF


            ELSE

                CALL DrawSpriteMask(Enemy(i%).x, Enemy(i%).y)

                Enemy(i%).x = Enemy(i%).x + deltaX%

                CALL DrawEnemy(i%)

            END IF

            
           

        ELSEIF Enemy(i%).isActive = FALSE AND Enemy(i%).IsRemoved = FALSE THEN
            'Inactive, count sleep ticks until active again

            Enemy(i%).CurSleepTicks = Enemy(i%).CurSleepTicks + 1

            IF Enemy(i%).CurSleepTicks > Enemy(i%).CurMaxSleepTicks THEN
                Enemy(i%).CurSleepTicks = 0
                Enemy(i%).isActive = TRUE
            END IF

        END IF

    NEXT i%

END SUB

SUB MovePlayer (deltaX%, deltaY%)


    ' Moves player the specified amount. Also checks for collisions with
    ' environment and calculates jump and pit Y changes.
    '
    ' This sub is the bane of my existence. It's not pretty... and needs
    ' to be re-done but it works well enough for now.


    'check new X for level out of bounds
    IF Player.x + deltaX% > OUTOFBOUNDSX OR Player.x + deltaX% < 0 THEN
        deltaX% = 0
    END IF



    ' Get current level grid location.

    leftLevelX% = INT((Player.x) / MAXSPRITEWIDTH) + 1
    rightLevelX% = INT((Player.x + MAXSPRITEWIDTH - 1) / MAXSPRITEWIDTH) + 1
   
    
    topLevelY% = INT(Player.y / MAXSPRITEHEIGHT) + 1
    bottomLevelY% = INT((Player.y + MAXSPRITEHEIGHT) / MAXSPRITEHEIGHT) + 1

   
    ' DEBUG DATA
    'PSET (rightLevelX% * MAXSPRITEWIDTH + 1, topLevelY% * MAXSPRITEHEIGHT + 1), 12
    'PSET (rightLevelX% * MAXSPRITEWIDTH + 1, bottomLevelY% * MAXSPRITEHEIGHT + 1 + MAXSPRITEHEIGHT), 12
  
    'LINE (leftLevelX% * MAXSPRITEWIDTH, topLevelY% * MAXSPRITEHEIGHT)-(rightLevelX% * MAXSPRITEWIDTH, bottomLevelY% * MAXSPRITEHEIGHT), 14, B

    'LOCATE 21
    'PRINT "LX:"; leftLevelX%; " RX:"; rightLevelX%; " TY:"; topLevelY%; " BY: "; bottomLevelY%


    ' Check if player is standing on the ground.

    IF bottomLevelY% <= UBOUND(LevelTileInfoMap, 2) AND bottomLevelY% > 0 THEN

        IF Player.IsJumping = FALSE AND (LevelTileInfoMap(leftLevelX%, bottomLevelY%).TypeId = TILETYPEBLOCKING OR LevelTileInfoMap(rightLevelX%, bottomLevelY%).TypeId = TILETYPEBLOCKING) THEN
           
            Player.IsOnGround = TRUE
           
            'LOCATE 23, 27
            'PRINT "G:T"

       
       
        'Check if player is standing on tree or sled marker.
        'really sort of gross... but i guess that fits this sub.

        ELSEIF Player.IsJumping = FALSE AND (LevelTileInfoMap(leftLevelX%, bottomLevelY%).TypeId = TILETYPEMARKER OR LevelTileInfoMap(rightLevelX%, bottomLevelY%).TypeId = TILETYPEMARKER) THEN
           
            IF IsBlockingMarkerTile%(leftLevelX%, bottomLevelY%) OR IsBlockingMarkerTile%(rightLevelX%, bottomLevelY%) THEN
                Player.IsOnGround = TRUE
            ELSE
                deltaY% = deltaY% + 2
            END IF

          

        ELSE
            Player.IsOnGround = FALSE
           
            'LOCATE 23, 27
            'PRINT "G:F"

            'If not on the ground, they're falling.
            deltaY% = deltaY% + 2


            'check if falling onto an item
            CALL CheckPickUpItem(leftLevelX%, bottomLevelY%)
            CALL CheckPickUpItem(rightLevelX%, bottomLevelY%)

   
        END IF


    END IF


    ' Upward jump change per game tick if jumping.
    IF Player.IsJumping THEN
        deltaY% = deltaY% - 6
    END IF



    'Recalculate Y sprite edges now that deltaY% has been finalized.
    topLevelY% = INT((Player.y + deltaY%) / MAXSPRITEHEIGHT) + 1
    bottomLevelY% = INT((Player.y + deltaY% + MAXSPRITEHEIGHT - 1) / MAXSPRITEHEIGHT) + 1



    ' Check right wall collisions and set sprite dir.
    IF deltaX% > 0 THEN

        Player.SpriteFrame.Direction = FORWARD

        checkLevelX% = INT((Player.x + MAXSPRITEWIDTH - 1 + deltaX%) / MAXSPRITEWIDTH) + 1

        'LINE (checkLevelX% * MAXSPRITEWIDTH, topLevelY% * MAXSPRITEHEIGHT)-(checkLevelX% * MAXSPRITEWIDTH, bottomLevelY% * MAXSPRITEHEIGHT), 9, B


        CALL CheckPickUpPresent(checkLevelX%, bottomLevelY%)
        CALL CheckDropOffPresent(checkLevelX%, bottomLevelY%)


        IF LevelTileInfoMap(checkLevelX%, topLevelY%).TypeId = TILETYPEBLOCKING OR LevelTileInfoMap(checkLevelX%, bottomLevelY%).TypeId = TILETYPEBLOCKING THEN
            deltaX% = 0
        ELSEIF LevelTileInfoMap(checkLevelX%, topLevelY%).TypeId = TILETYPEMARKER OR LevelTileInfoMap(checkLevelX%, bottomLevelY%).TypeId = TILETYPEMARKER THEN

            IF IsBlockingMarkerTile%(checkLevelX%, topLevelY%) OR IsBlockingMarkerTile%(checkLevelX%, bottomLevelY%) THEN
                deltaX% = 0
            END IF


        END IF
           
        CALL CheckPickUpItem(checkLevelX%, bottomLevelY%)

           

    END IF


    ' Check left wall collisions and set sprite dir.
    IF deltaX% < 0 THEN

        Player.SpriteFrame.Direction = BACKWARD

        checkLevelX% = INT((Player.x + deltaX%) / MAXSPRITEWIDTH) + 1

        'LINE (checkLevelX% * MAXSPRITEWIDTH, topLevelY% * MAXSPRITEHEIGHT)-(checkLevelX% * MAXSPRITEWIDTH, bottomLevelY% * MAXSPRITEHEIGHT), 9, B
       
   
        CALL CheckPickUpPresent(checkLevelX%, bottomLevelY%)
        CALL CheckDropOffPresent(checkLevelX%, bottomLevelY%)


        IF LevelTileInfoMap(checkLevelX%, topLevelY%).TypeId = TILETYPEBLOCKING OR LevelTileInfoMap(checkLevelX%, bottomLevelY%).TypeId = TILETYPEBLOCKING THEN
            deltaX% = 0
        ELSEIF LevelTileInfoMap(checkLevelX%, topLevelY%).TypeId = TILETYPEMARKER OR LevelTileInfoMap(checkLevelX%, bottomLevelY%).TypeId = TILETYPEMARKER THEN

            IF IsBlockingMarkerTile%(checkLevelX%, topLevelY%) OR IsBlockingMarkerTile%(checkLevelX%, bottomLevelY%) THEN
                deltaX% = 0
            END IF

        END IF

        CALL CheckPickUpItem(checkLevelX%, bottomLevelY%)

          

    END IF


    'check ceiling collisons
    IF deltaY% < 0 THEN

       
        IF LevelTileInfoMap(leftLevelX%, topLevelY%).TypeId = TILETYPEBLOCKING OR LevelTileInfoMap(rightLevelX%, topLevelY%).TypeId = TILETYPEBLOCKING THEN
            deltaY% = 0
            Player.IsJumping = FALSE
        END IF
        

    END IF


    'end jump
    IF Player.IsJumping AND Player.y + deltaY% <= Player.maxJumpY OR Player.IsOnGround = TRUE THEN
        Player.IsJumping = FALSE
    END IF


    
    'LOCATE 22, 32
    'PRINT "dY:"; deltaY%
    'LOCATE 23, 32
    'PRINT "dX:"; deltaX%
   
    'Don't bother drawing if nothing has changed.
    IF deltaX% = 0 AND deltaY% = 0 THEN EXIT SUB


    CALL DrawSpriteMask(Player.x, Player.y)
   

    Player.x = Player.x + deltaX%
    Player.y = Player.y + deltaY%


    CALL DrawFgTiles


    IF Player.y >= OUTOFBOUNDSY THEN
        Player.y = 0 'debug out of screen handling
                     'should remove a life and reset at start.

        CALL ChangePlayerHealth(-MAXHEALTH)
    END IF

    IF Player.y < 0 THEN
        Player.y = 0
        Player.IsJumping = FALSE
    END IF


    'LOCATE 20, 32
    'PRINT "PY:"; Player.y
    'LOCATE 21, 32
    'PRINT "PX:"; Player.x

   
  

   
    CALL DrawPlayer
   

END SUB

SUB PlaySfx (sfxId%)


    IF SoundOn = FALSE THEN
        EXIT SUB
    END IF


    SELECT CASE sfxId%

        CASE MENUSELECTSFX
            PLAY "O2 L32 MF >DBG"


        CASE JUMPSFX
            PLAY "O3 L32 MB C"


        CASE PICKUPITEMSFX
            PLAY "O2 L32 MB FA>C"


        CASE BLENDSFX
            PLAY "O2 L32 MB GFED"

        CASE UNBLENDSFX
            PLAY "O2 L32 MB DEFG"

        CASE RUNSFX
            PLAY "O2 L32 MB CEG"

        CASE WALKSFX
            PLAY "O2 L32 MB GEC"


        CASE PICKUPPRESENTSFX
            PLAY "O3 L42 MB CDEG"

        CASE DROPOFFPRESENTSFX
            PLAY "O3 L42 MB GEDC"

        CASE PLAYERDIEDSFX
            PLAY "O1 L28 MB ACFE"

        CASE PLAYERHURTSFX
            PLAY "O3 L42 MB ECA"

        CASE LEVELCOMPLETESFX
            PLAY "O1 MB L4 >C<B L8 A L4 G L8 F L4 EDC "
            

        CASE GAMEOVERSFX
            PLAY "O1 MB L8 C<BB- L2 A L4 A- L8 >D-"

            

    END SELECT

END SUB

SUB RunLevel
    quit% = FALSE
    userQuit% = FALSE


    isBleedFrame% = FALSE 'used for blending hurt damage
    bleedFrame% = 0
    enemyFrame% = 0

    ERASE KeyState
    LastKeyStateId = -1


    lastRunToggle! = 999
    lastBlendToggle! = 999
    lastSoundToggle! = 999

    toggleBounceThreshold! = .25



    WHILE quit% = FALSE
  
        start! = TIMER
 

        'in% = INP(96)


        'LOCATE 21, 22
        'PRINT "IN:"; in%

        pDeltaX% = 0
        pDeltaY% = 0


        'this gets called a bunch just to make sure the KeyState array
        'is up to date... it's not pretty by any means...
        CALL SetKeyState

       
        IF KeyState(STATEIDRIGHTARROW) THEN
            'Make sure key is still pressed to avoid
            'stuck keys. (Same with left arrow)

            keyValidate% = INP(96)
            IF keyValidate% <> 170 THEN
                pDeltaX% = Player.StepSize
            ELSE
                KeyState(STATEIDRIGHTARROW) = FALSE
            END IF
        END IF
        
       
        IF KeyState(STATEIDLEFTARROW) THEN
            keyValidate% = INP(96)
            IF keyValidate% <> 170 THEN
                pDeltaX% = -Player.StepSize
            ELSE
                KeyState(STATEIDLEFTARROW) = FALSE
            END IF

            
        END IF
       
        IF KeyState(STATEIDSPACEKEY) THEN

            KeyState(STATEIDSPACEKEY) = FALSE 'turn off to avoid any stuck key

            IF Player.IsJumping = FALSE AND Player.IsOnGround THEN

                CALL PlaySfx(JUMPSFX)

                Player.IsJumping = TRUE
                Player.IsOnGround = FALSE
                Player.maxJumpY = Player.y - MAXJUMPHEIGHT
            END IF
        END IF
       
        IF KeyState(STATEIDRKEY) THEN
            CALL Init(LevelInfo.LevelFileName)
        END IF

        IF KeyState(STATEIDQKEY) THEN
           
             'END    ' - for debug
             

            quit% = TRUE
            userQuit% = TRUE
            Player.IsGameOver = TRUE
        END IF


        'Running
        IF KeyState(STATEIDZKEY) AND Player.IsBlending = FALSE THEN

            KeyState(STATEIDZKEY) = FALSE

            IF TIMER - lastRunToggle! > toggleBounceThreshold! THEN

                lastRunToggle! = TIMER

                IF Player.IsRunning THEN

                    CALL PlaySfx(WALKSFX)

                    Player.IsRunning = FALSE
                    Player.StepSize = DEFAULTPLAYERSTEP
                ELSE
                    CALL PlaySfx(RUNSFX)

                    Player.IsRunning = TRUE
                    Player.IsBlending = FALSE

                    Player.StepSize = Player.StepSize * 2

                END IF
                CALL DrawStatusInfo(FALSE, FALSE, FALSE, FALSE, FALSE, TRUE)
            END IF
        END IF

        'Blending
        IF KeyState(STATEIDBKEY) THEN
            KeyState(STATEIDBKEY) = FALSE

            IF TIMER - lastBlendToggle! > toggleBounceThreshold! THEN

                lastBlendToggle! = TIMER


                IF Player.IsBlending THEN

                    CALL PlaySfx(UNBLENDSFX)

                    Player.IsBlending = FALSE
                    Player.StepSize = DEFAULTPLAYERSTEP
                ELSE

                    CALL PlaySfx(BLENDSFX)

                    Player.IsBlending = TRUE
              
              
                    'if running, put back in walk mode before recalculating
                    'step size for blend.
                    IF Player.IsRunning THEN
                        Player.IsRunning = FALSE
                        Player.StepSize = DEFAULTPLAYERSTEP
                    END IF
              
              
                    Player.StepSize = Player.StepSize / 2

                END IF
                CALL DrawPlayer
                CALL DrawStatusInfo(FALSE, FALSE, FALSE, FALSE, FALSE, TRUE)
            END IF
        END IF


        IF KeyState(STATEIDSKEY) THEN
            KeyState(STATEIDSKEY) = FALSE

            IF TIMER - lastSoundToggle! > toggleBounceThreshold! THEN

                lastSoundToggle! = TIMER


                CALL ToggleSound
            END IF

        END IF



        'SELECT CASE in%

        '    CASE QKEY
        '        quit% = TRUE
        '        userQuit% = TRUE
        '        Player.IsGameOver = TRUE

           
            ' DEBUG keys
            '
            'CASE UPARROW
            '    pDeltaY% = -Player.StepSize
            '
            'CASE DOWNARROW
            '    pDeltaY% = Player.StepSize


            'CASE RIGHTARROW
            '    pDeltaX% = Player.StepSize

            'CASE LEFTARROW
            '    pDeltaX% = -Player.StepSize

            'CASE SPACEKEY
            '    IF Player.IsJumping = FALSE AND Player.IsOnGround THEN

            '        CALL PlaySfx(JUMPSFX)

           '         Player.IsJumping = TRUE
            '        Player.IsOnGround = FALSE
            '        Player.maxJumpY = Player.y - MAXJUMPHEIGHT
           '     END IF


           ' CASE RKEY
           '     CALL Init(LevelInfo.LevelFileName)
           '

        'END SELECT



        ' Toggle keys seem to do better with INKEY$. They have too much
        ' "bounce" using INP(96).

        'inkeyVal$ = INKEY$
  
        'Running
        'IF UCASE$(inkeyVal$) = "Z" AND Player.IsBlending = FALSE THEN
        '    IF Player.IsRunning THEN

        '        CALL PlaySfx(WALKSFX)

        '        Player.IsRunning = FALSE
        '        Player.StepSize = DEFAULTPLAYERSTEP
        '    ELSE
        '        CALL PlaySfx(RUNSFX)

        '        Player.IsRunning = TRUE
        '        Player.IsBlending = FALSE

        '        Player.StepSize = Player.StepSize * 2
        '
        '    END IF
        '    CALL DrawStatusInfo(FALSE, FALSE, FALSE, FALSE, FALSE, TRUE)
            
        'Blending
        'ELSEIF UCASE$(inkeyVal$) = "B" THEN
        '    IF Player.IsBlending THEN
        '
        '        CALL PlaySfx(UNBLENDSFX)
        '
        '        Player.IsBlending = FALSE
        '        Player.StepSize = DEFAULTPLAYERSTEP
        '    ELSE
        '
        '        CALL PlaySfx(BLENDSFX)
        '
        '        Player.IsBlending = TRUE
        '
        '
        '        'if running, put back in walk mode before recalculating
        '        'step size for blend.
        '        IF Player.IsRunning THEN
        '            Player.IsRunning = FALSE
        '            Player.StepSize = DEFAULTPLAYERSTEP
        '        END IF
        '
        '
        '        Player.StepSize = Player.StepSize / 2
        '
        '    END IF
        '    CALL DrawPlayer
        '    CALL DrawStatusInfo(FALSE, FALSE, FALSE, FALSE, FALSE, TRUE)
        '
        'ELSEIF UCASE$(inkeyVal$) = "S" THEN
        '    CALL ToggleSound
        '
        'END IF

        'CALL ClearKbBuffer

        CALL SetKeyState



    '    CALL DrawDebugGrid



        IF enemyFrame% = ENEMYMOVEFRAME THEN
            CALL MoveEnemies
            enemyFrame% = 0
        ELSE
            enemyFrame% = enemyFrame% + 1
        END IF

        CALL SetKeyState

        CALL MovePlayer(pDeltaX%, pDeltaY%)

       
        CALL CheckEnemyPlayerCollisions


        'bleed health if walking while blending or moving in run mode
        IF pDeltaX% <> 0 AND (Player.IsBlending OR Player.IsRunning) AND isBleedFrame% THEN
            CALL ChangePlayerHealth(-1)
        END IF

   
        CALL HandleLvlTimer


        CALL SetKeyState

        ' blend damage every other frame

        bleedFrame% = bleedFrame + 1
        IF bleedFrame% = 4 THEN
            isBleedFrame% = TRUE
            bleedFrame% = 0
        ELSE
            isBleedFrame% = FALSE
        END IF


  
        'Goes back to main menu on game over or quit.
        IF Player.IsGameOver AND userQuit% = FALSE THEN
            CALL PlaySfx(GAMEOVERSFX)
            quit% = TRUE

            CALL ClearKbBuffer

            LINE (20, 25)-(232, 90), 12, BF
            LINE (22, 26)-(230, 88), 0, BF
             
            LOCATE 6, 11
            COLOR 15
            PRINT "GAME OVER"
            LOCATE 8, 5
            PRINT "TOTAL SCORE:"; Player.Score

            LOCATE 10, 5
            COLOR 7
            PRINT "PRESS ENTER TO CONTINUE"


            DO
            LOOP WHILE INKEY$ <> CHR$(13)


        END IF
      
        IF LevelInfo.LevelComplete THEN
           
            quit% = TRUE

            CALL HandleLevelComplete

        END IF

       
        CALL SetKeyState

        'allow screen time to refresh before updating.
        'not sure how much it's actually helping...
        'WAIT &H3DA, 8
        WAIT &H3DA, 8, 8 'Waits until draw finishes.. slows down a bit.
       

        CALL SetKeyState

        'fps control
        WHILE TIMER - start! <= .025
            
            CALL SetKeyState
            
        WEND

        CALL ClearKbBuffer



    WEND

END SUB

SUB SetKeyState

    ' Sets key state flags in the KeyState array

    kIn% = INP(96)

    'inPrintY = inPrintY + 1
    'IF inPrintY > 23 THEN inPrintY = 15

    'IF kIn% = 170 THEN COLOR kIn% - 10

    'LOCATE inPrintY, 22
    'PRINT "IN:"; kIn%



    'stateIdx% = -1

    IF kIn% THEN
        SELECT CASE kIn%

            CASE LEFTARROW

                KeyState(STATEIDLEFTARROW) = TRUE
                KeyState(STATEIDRIGHTARROW) = FALSE
                LastKeyStateId = STATEIDLEFTARROW

            CASE LEFTARROW + 128
                KeyState(STATEIDLEFTARROW) = FALSE


            CASE RIGHTARROW
                KeyState(STATEIDRIGHTARROW) = TRUE
                KeyState(STATEIDLEFTARROW) = FALSE
                LastKeyStateId = STATEIDRIGHTARROW

           
            CASE RIGHTARROW + 128
                KeyState(STATEIDRIGHTARROW) = FALSE


            CASE SPACEKEY
                KeyState(STATEIDSPACEKEY) = TRUE
                LastKeyStateId = STATEIDSPACEKEY

            CASE SPACEKEY + 128
                KeyState(STATEIDSPACEKEY) = FALSE


            CASE QKEY
                KeyState(STATEIDQKEY) = TRUE
                LastKeyStateId = STATEIDQKEY

            CASE QKEY + 128
                KeyState(STATEIDQKEY) = FALSE


            CASE RKEY
                KeyState(STATEIDRKEY) = TRUE
                LastKeyStateId = STATEIDRKEY

            CASE RKEY + 128
                KeyState(STATEIDRKEY) = FALSE

            CASE BKEY
                KeyState(STATEIDBKEY) = TRUE
                LastKeyStateId = STATEIDBKEY

            CASE BKEY + 128
                KeyState(STATEIDBKEY) = FALSE

            CASE ZKEY
                LastKeyStateId = STATEIDZKEY
                KeyState(STATEIDZKEY) = TRUE

            CASE ZKEY + 128
                KeyState(STATEIDZKEY) = FALSE

            CASE SKEY
                KeyState(STATEIDSKEY) = TRUE
                LastKeyStateId = STATEIDSKEY

            CASE SKEY + 128
                KeyState(STATEIDSKEY) = FALSE


            CASE 170
                ' Causes left/right to disengage on jump
                ' without, quick taps gets directions stuck on

                'IF LastKeyStateId >= LBOUND(KeyState) THEN
                '    KeyState(LastKeyStateId) = FALSE
                'END IF
            '    ERASE KeyState


        END SELECT
    END IF

    'IF stateIdx% < 0 THEN EXIT SUB




    'IF kIn% THEN
    '    IF kIn% < UBOUND(KeyState) THEN

    '        KeyState(kIn%) = TRUE
    '        LastKeyState = kIn%


            'Force these to disable each other in the case
            'that a key press is missed.
    '        SELECT CASE kIn%
    '            CASE LEFTARROW
    '                KeyState(RIGHTARROW) = FALSE
    '
    '            CASE RIGHTARROW
    '                KeyState(LEFTARROW) = FALSE
            
    '        END SELECT

            

    '    ELSE
    '        KeyState(kIn% - UBOUND(KeyState)) = FALSE
    '        IF kIn% = 170 THEN 'release
    '            KeyState(LastKeyState) = FALSE
    '        END IF
    '    END IF
    'END IF


'    CALL ClearKbBuffer
   

END SUB

SUB SetSpriteForPut (SpriteId%, destSprite())

    CLS


    'DrawSprite vs Get are off by 1x1 px.

    CALL DrawSprite(0, 0, SpriteId%)

    GET (1, 1)-(MAXSPRITEWIDTH, MAXSPRITEHEIGHT), destSprite
  
   
    ' DEBUG just to verify sprite boxes match
    'SLEEP
    'LINE (1, 1)-(MAXSPRITEWIDTH, MAXSPRITEHEIGHT), 15, B
    'SLEEP
   
    CLS

END SUB

SUB ToggleSound

    IF SoundOn THEN
        SoundOn = FALSE
    ELSE
        SoundOn = TRUE
    END IF

END SUB

